# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
The AffineTransform class
"""

import numpy as np

from .transform import Transform
from .affine_utils import apply_affine

################################################################################
# Class `AffineTransform`
################################################################################
class AffineTransform(Transform):
    """
    A transformation from an input 3D space to an output 3D space defined
    by an affine matrix.

    It is defined by the affine matrix , and the name of the input and output 
    spaces.
    """

    # The coordinate mapping from input space to output space
    affine = None

    def __init__(self, input_space, output_space, affine):
        """ Create a new affine transform object.

            Parameters
            ----------

            input_space: string
                Name of the input space
            output_space: string
                Name of the output space
            affine: 4x4 ndarray 
                Affine matrix giving the coordinate mapping between the 
                input and output space. 
        """
        assert hasattr(affine, '__array__'), \
            'affine argument should be an array-like'
        self.affine       = affine 
        self.input_space  = input_space
        self.output_space = output_space

    #-------------------------------------------------------------------------
    # Transform Interface
    #-------------------------------------------------------------------------

    def composed_with(self, transform):
        """ Returns a new transform obtained by composing this transform
            with the one provided.

            Parameters
            -----------
            transform: nipy.core.transforms.transform object
                The transform to compose with.
        """
        if not isinstance(transform, AffineTransform):
            return super(AffineTransform, self).composed_with(transform)
        self._check_composition(transform)
        new_affine = np.dot(transform.affine, self.affine)
        return AffineTransform(self.input_space, 
                               transform.output_space,
                               new_affine, 
                               )


    def get_inverse(self):
        """ Return the inverse transform.
        """
        return AffineTransform(self.output_space,
                               self.input_space,
                               np.linalg.inv(self.affine),
                               )

    def inverse_mapping(self, x, y, z):
        """ Transform the given coordinate from output space to input space.

            Parameters
            ----------
            x: number or ndarray
                The x coordinates
            y: number or ndarray
                The y coordinates
            z: number or ndarray
                The z coordinates
        """
        return apply_affine(x, y, z, np.linalg.inv(self.affine))


    def mapping(self, x, y, z):
        """ Transform the given coordinate from input space to output space.

            Parameters
            ----------
            x: number or ndarray
                The x coordinates
            y: number or ndarray
                The y coordinates
            z: number or ndarray
                The z coordinates
        """
        return apply_affine(x, y, z, self.affine)


    #---------------------------------------------------------------------------
    # Private methods
    #---------------------------------------------------------------------------

    def __repr__(self):
        representation = \
                '%s(\n  affine=%s,\n  input_space=%s,\n  output_space=%s)' % (
                self.__class__.__name__,
                '\n         '.join(repr(self.affine).split('\n')),
                self.input_space,
                self.output_space, 
                )
        return representation


    def __copy__(self):
        """ Copy the transform 
        """
        return self.__class__(affine=self.affine,
                              input_space=self.input_space,
                              output_space=self.output_space)


    def __deepcopy__(self, option):
        """ Copy the Image and the arrays and metadata it contains.
        """
        return self.__class__(affine=self.affine.copy(),
                              input_space=self.input_space,
                              output_space=self.output_space)


    def __eq__(self, other):
        return (isinstance(other, self.__class__)
                and self.input_space == other.input_space
                and self.output_space == other.output_space
                and np.allclose(self.affine, other.affine))






# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
try: # python 2
    from ConfigParser import ConfigParser
except ImportError: # python 3
    from configparser import ConfigParser

import os

NIPY_DEFAULTS = dict()

################################################################################
def get_nipy_info():
    """ Reuse numpy's distutils to get and store information about nipy
        in the site.cfg.
    """
    from numpy.distutils.system_info import get_standard_file
    files = get_standard_file('site.cfg')
    cp = ConfigParser(NIPY_DEFAULTS)
    cp.read(files)
    if not cp.has_section('nipy'):
        cp.add_section('nipy')
    info = dict(cp.items('nipy'))
    for key, value in info.items():
        if value.startswith('~'):
            info[key] = os.path.expanduser(value)
    # Ugly fix for bug 409269
    if 'libraries' in info and isinstance(info['libraries'], basestring):
        info['libraries'] = [info['libraries']]
    # End of ugly fix
    return info


################################################################################
def configuration(parent_package='',top_path=None):
    from numpy.distutils.misc_util import Configuration
    from numpy.distutils.system_info import system_info
    config = Configuration('nipy', parent_package, top_path)

    # List all packages to be loaded here
    config.add_subpackage('algorithms')
    config.add_subpackage('interfaces')
    config.add_subpackage('core')
    config.add_subpackage('fixes')
    config.add_subpackage('io')
    config.add_subpackage('modalities')
    config.add_subpackage('utils')
    config.add_subpackage('tests')
    config.add_subpackage('externals')
    config.add_subpackage('testing')

    # Note: this is a special subpackage containing that will later be
    # migrated to whichever parts of the main package they logically
    # belong in. But initially we are putting everythin under this
    # subpackage to make the management and migration easier.
    config.add_subpackage('labs')

    #####################################################################
    # Store the setup information, including the nipy-specific
    # information in a __config__ file.
    class nipy_info(system_info):
        """ We are subclassing numpy.distutils's system_info to
            insert information in the __config__ file.
            The class name determines the name of the variable
            in the __config__ file.
        """
    nipy_info().set_info(**get_nipy_info())
    config.make_config_py()

    return config


if __name__ == '__main__':
    from numpy.distutils.core import setup
    setup(**configuration(top_path='').todict())



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
This module provides classes and definitions for using full width at half
maximum (FWHM) to be used in conjunction with Gaussian Random Field Theory
to determine resolution elements (resels).

A resolution element (resel) is defined as a block of pixels of the same
size as the FWHM of the smoothed image.

There are two methods implemented to estimate (3d, or volumewise) FWHM
based on a 4d Image:

    fastFHWM: used if the entire 4d Image is available
    iterFWHM: used when 4d Image is being filled in by slices of residuals
    
"""

__docformat__ = 'restructuredtext'

import numpy as np
from numpy.linalg import det

from nipy.core.api import Image

from .utils.matrices import pos_recipr

class Resels(object):
    """The Resels class.
    """
    def __init__(self, coordmap, normalized=False, fwhm=None, resels=None,
                 mask=None, clobber=False, D=3):
        """ Initialize resels class

        Parameters
        ----------
        coordmap : ``CoordinateMap``
                CoordinateMap over which fwhm and resels are to be estimated.
                Used in fwhm/resel conversion.
        fwhm : ``Image``
            Optional Image of FWHM. Used to convert
            FWHM Image to resels if FWHM is not being estimated.
        resels : ``Image``
            Optional Image of resels. Used to
            compute resels within a mask, for instance, if
            FWHM has already been estimated.
        mask : ``Image``
            Mask over which to integrate resels.
        clobber : ``bool``
            Clobber output FWHM and resel images?
        D : ``int``
            Can be 2 or 3, the dimension of the final volume.
        """
        self.fwhm = fwhm
        self.resels = resels
        self.mask = mask
        self.clobber = clobber
        self.coordmap = coordmap
        self.D = D
        self.normalized = normalized
        _transform = self.coordmap.affine
        self.wedge = np.power(np.fabs(det(_transform)), 1./self.D)

    def integrate(self, mask=None):
        """ Integrate resels within `mask` (or use self.mask)

        Parameters
        ----------
        mask : ``Image``
                Optional mask over which to integrate (add) resels.

        Returns
        -------
        total_resels :
            the resels contained in the mask
        FWHM : float
            an estimate of FWHM based on the average resel per voxel
        nvoxel: int
            the number of voxels in the mask
        """
        _resels = self.resels[:]
        if mask is not None:
            _mask = mask
        else:
            _mask = self.mask
        if _mask is not None:
            _mask = _mask[:].astype(np.int32)
            nvoxel = _mask.sum()
        else:
            _mask = 1.
            nvoxel = _resels.size
        _resels = (_resels * _mask).sum()
        _fwhm = self.resel2fwhm(_resels / nvoxel)
        return _resels, _fwhm, nvoxel

    def resel2fwhm(self, resels):
        """ Convert resels as `resels` to isotropic FWHM

        Parameters
        ----------
        resels : float
            Convert a resel value to an equivalent isotropic FWHM based on
            step sizes in self.coordmap.

        Returns
        -------
        fwhm : float
        """
        return np.sqrt(4*np.log(2.)) * self.wedge * pos_recipr(np.power(resels, 1./self.D))

    def fwhm2resel(self, fwhm):
        """ Convert FWHM `fwhm` to equivalent reseels per voxel

        Parameters
        ----------
        fwhm : float
            Convert an FWHM value to an equivalent resels per voxel based on
            step sizes in self.coordmap.


        Returns
        -------
        resels : float
        """
        return pos_recipr(np.power(fwhm / np.sqrt(4*np.log(2)) * self.wedge, self.D))

    def __iter__(self):
        """ Return iterator

        Returns
        -------
        itor : iterator
            self
        """
        if not self.fwhm:
            im = Image(np.zeros(self.resid.shape), coordmap=self.coordmap)
        else:
            im = \
              Image(self.fwhm, clobber=self.clobber, mode='w', coordmap=self.coordmap)
        self.fwhm = im

        if not self.resels:
            im = Image(np.zeros(self.resid.shape), coordmap=self.coordmap)
        else:
            im = \
              Image(self.resels, clobber=self.clobber, mode='w', coordmap=self.coordmap)
        self.resels = im

        return self


class ReselImage(Resels):

    def __init__(self, resels=None, fwhm=None, **keywords):
        """ Initialize resel image

        Parameters
        ----------
        resels : `core.api.Image`
            Image of resel per voxel values.
        fwhm : `core.api.Image`
            Image of FWHM values.
        keywords : ``dict``
            Passed as keywords arguments to `core.api.Image`
        """
        if not resels and not fwhm:
            raise ValueError, 'need either a resels image or an FWHM image'

        if fwhm is not None:
            fwhm = Image(fwhm, **keywords)
            Resels.__init__(self, fwhm, resels=resels, fwhm=fwhm)

        if resels is not None:
            resels = Image(resels, **keywords)
            Resels.__init__(self, resels, resels=resels, fwhm=fwhm)

        if not self.fwhm:
            self.fwhm = Image(self.resel2fwhm(self.resels[:]),
                              coordmap=self.coordmap, **keywords)

        if not self.resels:
            self.resels = Image(self.fwhm2resel(self.fwhm[:]),
                                coordmap=self.coordmap, **keywords)

    def __iter__(self):
        """ Return iterator

        Returns
        -------
        itor : iterator
            ``self``
        """
        return self


def _calc_detlam(xx, yy, zz, yx, zx, zy):
    """
    Calculate determinant of symmetric 3x3 matrix

    [[xx,yx,xz],
     [yx,yy,zy],
     [zx,zy,zz]]
     """
    
    return zz * (yy*xx - yx**2) - \
           zy * (zy*xx - zx*yx) + \
           zx * (zy*yx - zx*yy)



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import tempfile

import numpy as np

from nose import SkipTest
try:
    import matplotlib as mp
    # Make really sure that we don't try to open an Xserver connection.
    mp.use('svg', warn=False)
    import pylab as pl
    pl.switch_backend('svg')
except ImportError:
    raise SkipTest('Could not import matplotlib')

from ..activation_maps import demo_plot_map, plot_anat, plot_map
from ..anat_cache import mni_sform, _AnatCache



def test_demo_plot_map():
    # This is only a smoke test
    mp.use('svg', warn=False)
    import pylab as pl
    pl.switch_backend('svg')
    demo_plot_map()
    # Test the black background code path
    demo_plot_map(black_bg=True)


def test_plot_anat():
    # This is only a smoke test
    mp.use('svg', warn=False)
    import pylab as pl
    pl.switch_backend('svg')
    data = np.zeros((20, 20, 20))
    data[3:-3, 3:-3, 3:-3] = 1
    ortho_slicer = plot_anat(data, mni_sform, dim=True)
    ortho_slicer = plot_anat(data, mni_sform, cut_coords=(80, -120, -60))
    # Saving forces a draw, and thus smoke-tests the axes locators
    pl.savefig(tempfile.TemporaryFile())
    ortho_slicer.edge_map(data, mni_sform, color='c')

    # Test saving with empty plot
    z_slicer = plot_anat(anat=False, slicer='z')
    pl.savefig(tempfile.TemporaryFile())
    z_slicer = plot_anat(slicer='z')
    pl.savefig(tempfile.TemporaryFile())
    z_slicer.edge_map(data, mni_sform, color='c')
    # Smoke test coordinate finder, with and without mask
    plot_map(np.ma.masked_equal(data, 0), mni_sform, slicer='x')
    plot_map(data, mni_sform, slicer='y')


def test_anat_cache():
    # A smoke test, that can work only if the templates are installed
    try:
        _AnatCache.get_blurred()
    except OSError:
        "The templates are not there"
        pass





# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
This module provides a class for principal components analysis (PCA).

PCA is an orthonormal, linear transform (i.e., a rotation) that maps the
data to a new coordinate system such that the maximal variability of the
data lies on the first coordinate (or the first principal component), the
second greatest variability is projected onto the second coordinate, and
so on.  The resulting data has unit covariance (i.e., it is decorrelated).
This technique can be used to reduce the dimensionality of the data.

More specifically, the data is projected onto the eigenvectors of the
covariance matrix.
"""

import numpy as np
import scipy.linalg as spl

from ...core.image.image import rollimg
from ...core.reference.coordinate_map import (io_axis_indices, orth_axes,
                                              drop_io_dim, AxisError)


def pca(data, axis=0, mask=None, ncomp=None, standardize=True,
        design_keep=None, design_resid='mean', tol_ratio=0.01):
    """Compute the SVD PCA of an array-like thing over `axis`.

    Parameters
    ----------
    data : ndarray-like (np.float)
       The array on which to perform PCA over axis `axis` (below)
    axis : int, optional
       The axis over which to perform PCA (axis identifying
       observations).  Default is 0 (first)
    mask : ndarray-like (np.bool), optional
       An optional mask, should have shape given by data axes, with
       `axis` removed, i.e.: ``s = data.shape; s.pop(axis);
       msk_shape=s``
    ncomp : {None, int}, optional
       How many component basis projections to return. If ncomp is None
       (the default) then the number of components is given by the
       calculated rank of the data, after applying `design_keep`,
       `design_resid` and `tol_ratio` below.  We always return all the
       basis vectors and percent variance for each component; `ncomp`
       refers only to the number of basis_projections returned.
    standardize : bool, optional
       If True, standardize so each time series (after application of
       `design_keep` and `design_resid`) has the same standard
       deviation, as calculated by the ``np.std`` function.
    design_keep : None or ndarray, optional
       Data is projected onto the column span of design_keep.
       None (default) equivalent to ``np.identity(data.shape[axis])``
    design_resid : str or None or ndarray, optional
       After projecting onto the column span of design_keep, data is
       projected perpendicular to the column span of this matrix.  If
       None, we do no such second projection.  If a string 'mean', then
       the mean of the data is removed, equivalent to passing a column
       vector matrix of 1s.
    tol_ratio : float, optional
       If ``XZ`` is the vector of singular values of the projection
       matrix from `design_keep` and `design_resid`, and S are the
       singular values of ``XZ``, then `tol_ratio` is the value used to
       calculate the effective rank of the projection of the design, as
       in ``rank = ((S / S.max) > tol_ratio).sum()``

    Returns
    -------
    results : dict
        $G$ is the number of non-trivial components found after applying
       `tol_ratio` to the projections of `design_keep` and
       `design_resid`.

       `results` has keys:

       * ``basis_vectors``: series over `axis`, shape (data.shape[axis], G) -
          the eigenvectors of the PCA
       * ``pcnt_var``: percent variance explained by component, shape
          (G,)
       * ``basis_projections``: PCA components, with components varying
          over axis `axis`; thus shape given by: ``s = list(data.shape);
          s[axis] = ncomp``
       * ``axis``: axis over which PCA has been performed.

    Notes
    -----
    See ``pca_image.m`` from ``fmristat`` for Keith Worsley's code on
    which some of this is based.

    See: http://en.wikipedia.org/wiki/Principal_component_analysis for
    some inspiration for naming - particularly 'basis_vectors' and
    'basis_projections'

    Examples
    --------
    >>> arr = np.random.normal(size=(17, 10, 12, 14))
    >>> msk = np.all(arr > -2, axis=0)
    >>> res = pca(arr, mask=msk, ncomp=9)

    Basis vectors are columns.  There is one column for each component.  The
    number of components is the calculated rank of the data matrix after
    applying the various projections listed in the parameters.  In this case we
    are only removing the mean, so the number of components is one less than the
    axis over which we do the PCA (here axis=0 by default).

    >>> res['basis_vectors'].shape
    (17, 16)

    Basis projections are arrays with components in the dimension over which we
    have done the PCA (axis=0 by default).  Because we set `ncomp` above, we
    only retain `ncomp` components.

    >>> res['basis_projections'].shape
    (9, 10, 12, 14)
    """
    data = np.asarray(data)
    # We roll the PCA axis to be first, for convenience
    if axis is None:
        raise ValueError('axis cannot be None')
    data = np.rollaxis(data, axis)
    if mask is not None:
        mask = np.asarray(mask)
        if not data.shape[1:] == mask.shape:
            raise ValueError('Mask should match dimensions of data other than '
                             'the axis over which to do the PCA')
    if design_resid == 'mean':
        # equivalent to: design_resid = np.ones((data.shape[0], 1))
        def project_resid(Y):
            return Y - Y.mean(0)[None,...]
    elif design_resid is None:
        def project_resid(Y): return Y
    else: # matrix passed, we hope
        projector = np.dot(design_resid, spl.pinv(design_resid))
        def project_resid(Y):
            return Y - np.dot(projector, Y)
    if standardize:
        def rmse_scales_func(std_source):
            # modifies array in place
            resid = project_resid(std_source)
            # root mean square of the residual
            rmse = np.sqrt(np.square(resid).sum(axis=0) / resid.shape[0])
            # positive 1/rmse
            return np.where(rmse<=0, 0, 1. / rmse)
    else:
        rmse_scales_func = None
    """
    Perform the computations needed for the PCA.  This stores the
    covariance/correlation matrix of the data in the attribute 'C'.  The
    components are stored as the attributes 'components', for an fMRI
    image these are the time series explaining the most variance.

    Now, we compute projection matrices. First, data is projected onto
    the columnspace of design_keep, then it is projected perpendicular
    to column space of design_resid.
    """
    if design_keep is None:
        X = np.eye(data.shape[0])
    else:
        X = np.dot(design_keep, spl.pinv(design_keep))
    XZ = project_resid(X)
    UX, SX, VX = spl.svd(XZ, full_matrices=0)
    # The matrix UX has orthonormal columns and represents the
    # final "column space" that the data will be projected onto.
    rank = (SX/SX.max() > tol_ratio).sum()
    UX = UX[:,:rank].T
    # calculate covariance matrix in full-rank column space.  The returned
    # array is roughly: YX = dot(UX, data); C = dot(YX, YX.T), perhaps where the
    # data has been standarized, perhaps summed over slices
    C_full_rank  = _get_covariance(data, UX, rmse_scales_func, mask)
    # find the eigenvalues D and eigenvectors Vs of the covariance
    # matrix
    D, Vs = spl.eigh(C_full_rank)
    # Compute basis vectors in original column space
    basis_vectors = np.dot(UX.T, Vs).T
    # sort both in descending order of eigenvalues
    order = np.argsort(-D)
    D = D[order]
    basis_vectors = basis_vectors[order]
    pcntvar = D * 100 / D.sum()
    """
    Output the component basis_projections
    """
    if ncomp is None:
        ncomp = rank
    subVX = basis_vectors[:ncomp]
    out = _get_basis_projections(data, subVX, rmse_scales_func)
    # Roll PCA image axis back to original position in data array
    if axis < 0:
        axis += data.ndim
    out = np.rollaxis(out, 0, axis+1)
    return {'basis_vectors': basis_vectors.T,
            'pcnt_var': pcntvar,
            'basis_projections': out,
            'axis': axis}


def _get_covariance(data, UX, rmse_scales_func, mask):
    # number of points in PCA dimension
    rank, n_pts = UX.shape
    C = np.zeros((rank, rank))
    # nan_to_num only for floating point masks
    if not mask is None:
        nan_to_num = mask.dtype.type in (np.sctypes['float'] +
                                         np.sctypes['complex'])
    # loop over next dimension to save memory
    if data.ndim == 2:
        # If we have 2D data, just do the covariance all in one shot, by using
        # a slice that is the equivalent of the ':' slice syntax
        slices = [slice(None)]
    else:
        # If we have more then 2D, then we iterate over slices in the second
        # dimension, in order to save memory
        slices = [slice(i,i+1) for i in range(data.shape[1])]
    for i, s_slice in enumerate(slices):
        Y = data[:,s_slice].reshape((n_pts, -1))
        # project data into required space
        YX = np.dot(UX, Y)
        if rmse_scales_func is not None:
            YX *= rmse_scales_func(Y)
        if mask is not None:
            # weight data with mask.  Usually the weights will be 0,1
            msk_slice = mask[s_slice].reshape(Y.shape[1])
            if nan_to_num: # but if floats, check for NaNs too.
                msk_slice = np.nan_to_num(msk_slice)
            YX = YX * msk_slice
        C += np.dot(YX, YX.T)
    return C


def _get_basis_projections(data, subVX, rmse_scales_func):
    ncomp = subVX.shape[0]
    out = np.empty((ncomp,) + data.shape[1:], np.float)
    for i in range(data.shape[1]):
        Y = data[:,i].reshape((data.shape[0], -1))
        U = np.dot(subVX, Y)
        if rmse_scales_func is not None:
            U *= rmse_scales_func(Y)
        U.shape = (U.shape[0],) + data.shape[2:]
        out[:,i] = U
    return out


def pca_image(img, axis='t', mask=None, ncomp=None, standardize=True,
              design_keep=None, design_resid='mean', tol_ratio=0.01):
    """ Compute the PCA of an image over a specified axis

    Parameters
    ----------
    img : Image
        The image on which to perform PCA over the given `axis`
    axis : str or int
        Axis over which to perform PCA. Default is 't'. If `axis` is an integer,
        gives the index of the input (domain) axis of `img`. If `axis` is a str, can be
        an input (domain) name, or an output (range) name, that maps to an input
        (domain) name.
    mask : Image, optional
        An optional mask, should have shape == image.shape[:3] and the same
        coordinate map as `img` but with `axis` dropped
    ncomp : {None, int}, optional
       How many component basis projections to return. If ncomp is None
       (the default) then the number of components is given by the
       calculated rank of the data, after applying `design_keep`,
       `design_resid` and `tol_ratio` below.  We always return all the
       basis vectors and percent variance for each component; `ncomp`
       refers only to the number of basis_projections returned.
    standardize : bool, optional
       If True, standardize so each time series (after application of
       `design_keep` and `design_resid`) has the same standard
       deviation, as calculated by the ``np.std`` function.
    design_keep : None or ndarray, optional
       Data is projected onto the column span of design_keep.
       None (default) equivalent to ``np.identity(data.shape[axis])``
    design_resid : str or None or ndarray, optional
       After projecting onto the column span of design_keep, data is
       projected perpendicular to the column span of this matrix.  If
       None, we do no such second projection.  If a string 'mean', then
       the mean of the data is removed, equivalent to passing a column
       vector matrix of 1s.
    tol_ratio : float, optional
       If ``XZ`` is the vector of singular values of the projection
       matrix from `design_keep` and `design_resid`, and S are the
       singular values of ``XZ``, then `tol_ratio` is the value used to
       calculate the effective rank of the projection of the design, as
       in ``rank = ((S / S.max) > tol_ratio).sum()``

    Returns
    -------
    results : dict
        $L$ is the number of non-trivial components found after applying
       `tol_ratio` to the projections of `design_keep` and
       `design_resid`.

       `results` has keys:
       * ``basis_vectors``: series over `axis`, shape (data.shape[axis], L) -
          the eigenvectors of the PCA
       * ``pcnt_var``: percent variance explained by component, shape
          (L,)
       * ``basis_projections``: PCA components, with components varying
          over axis `axis`; thus shape given by: ``s = list(data.shape);
          s[axis] = ncomp``
       * ``axis``: axis over which PCA has been performed.

    Examples
    --------
    >>> from nipy.testing import funcfile
    >>> from nipy import load_image
    >>> func_img = load_image(funcfile)

    Time is the fourth axis

    >>> func_img.coordmap.function_range
    CoordinateSystem(coord_names=('aligned-x=L->R', 'aligned-y=P->A', 'aligned-z=I->S', 't'), name='aligned', coord_dtype=float64)
    >>> func_img.shape
    (17, 21, 3, 20)

    Calculate the PCA over time, by default

    >>> res = pca_image(func_img)
    >>> res['basis_projections'].coordmap.function_range
    CoordinateSystem(coord_names=('aligned-x=L->R', 'aligned-y=P->A', 'aligned-z=I->S', 'PCA components'), name='aligned', coord_dtype=float64)

    The number of components is one less than the number of time points

    >>> res['basis_projections'].shape
    (17, 21, 3, 19)
    """
    img_klass = img.__class__
    # Which axes are we operating over?
    in_ax, out_ax = io_axis_indices(img.coordmap, axis)
    if None in (in_ax, out_ax):
        raise AxisError('Cannot identify matching input output axes with "%s"'
                        % axis)
    if not orth_axes(in_ax, out_ax, img.coordmap.affine):
        raise AxisError('Input and output axes found from "%s" not othogonal '
                        'to rest of affine' % axis)
    # Roll the chosen axis to input position zero
    work_img = rollimg(img, axis)
    if mask is not None:
        if not mask.coordmap.similar_to(drop_io_dim(img.coordmap, axis)):
            raise ValueError("Mask should have matching coordmap to `img` "
                             "coordmap with dropped axis %s" % axis)
    data = work_img.get_data()
    if mask is not None:
        mask_data = mask.get_data()
    else:
        mask_data = None
    # do the PCA
    res = pca(data, 0, mask_data, ncomp, standardize,
              design_keep, design_resid, tol_ratio)
    # Clean up images after PCA
    # Rename the axis we dropped, at position 0 after rollimg
    output_coordmap = work_img.coordmap.renamed_domain(
        {0: 'PCA components'})
    # And the matching output axis - which has not moved position
    output_coordmap = output_coordmap.renamed_range(
        {out_ax: 'PCA components'})
    output_img = img_klass(res['basis_projections'], output_coordmap)
    # We have to roll the axis back to the original position
    output_img = rollimg(output_img, 0, in_ax + 1)
    key = 'basis_vectors over %s' % axis
    res[key] = res['basis_vectors']
    res['basis_projections'] = output_img
    # Signal the roll in results
    res['axis'] = in_ax
    return res



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import numpy as np

from nibabel.affines import from_matvec

from ..pca import pca_image, pca as pca_array
from ....core.api import Image, AffineTransform, CoordinateSystem as CS
from ....core.reference.coordinate_map import (product as cm_product,
                                               drop_io_dim, AxisError)
from ....core.image.image import rollimg
from ....io.api import  load_image

from nose.tools import assert_raises
from numpy.testing import (assert_equal, assert_almost_equal,
                           assert_array_equal)
from ....testing import funcfile
from .test_pca import res2pos1

data_dict = {}

def setup():
    img = load_image(funcfile)
    # Here, I'm just doing this so I know that img.shape[0] is the number of
    # volumes
    t0_img = rollimg(img, 't')
    data_dict['nimages'] = t0_img.shape[0]
    # Below, I am just making a mask because I already have img, I know I can do
    # this. In principle, though, the pca function will just take another Image
    # as a mask
    img_data = t0_img.get_data()
    mask_cmap = drop_io_dim(img.coordmap, 't')
    first_frame = img_data[0]
    mask = Image(np.greater(first_frame, 500).astype(np.float64),
                 mask_cmap)
    data_dict['fmridata'] = img
    data_dict['mask'] = mask

    # print data_dict['mask'].shape, np.sum(data_dict['mask'].get_data())
    assert_equal(data_dict['mask'].shape, (17, 21, 3))
    assert_almost_equal(np.sum(data_dict['mask'].get_data()), 1071.0)

def _rank(p):
    return p['basis_vectors'].shape[1]


def test_PCAMask():
    nimages = data_dict['nimages']
    ntotal = nimages - 1
    ncomp = 5
    p = pca_image(data_dict['fmridata'], 't',
                  data_dict['mask'], ncomp=ncomp)
    assert_equal(_rank(p), ntotal)
    assert_equal(p['axis'], 3)
    assert_equal(p['basis_vectors over t'].shape, (nimages, ntotal))
    assert_equal(p['basis_projections'].shape, data_dict['mask'].shape +
                 (ncomp,))
    assert_equal(p['pcnt_var'].shape, (ntotal,))
    assert_almost_equal(p['pcnt_var'].sum(), 100.)
    assert_equal(p['basis_projections'].axes.coord_names,
                 ['i','j','k','PCA components'])
    assert_equal(p['basis_projections'].coordmap.affine,
                 data_dict['fmridata'].coordmap.affine)


def test_mask_match():
    # we can't do PCA over spatial axes if we use a spatial mask
    ncomp = 5
    out_coords = data_dict['mask'].reference.coord_names
    for i, o, n in zip('ijk', out_coords, [0,1,2]):
        assert_raises(ValueError,
                      pca_image,
                      data_dict['fmridata'],
                      i,
                      data_dict['mask'],
                      ncomp)
        assert_raises(ValueError,
                      pca_image,
                      data_dict['fmridata'],
                      o,
                      data_dict['mask'],
                      ncomp)
        assert_raises(ValueError,
                      pca_image,
                      data_dict['fmridata'],
                      n,
                      data_dict['mask'],
                      ncomp)


def test_PCAMask_nostandardize():
    nimages = data_dict['nimages']
    ntotal = nimages - 1
    ncomp = 5
    p = pca_image(data_dict['fmridata'], 't',
                  data_dict['mask'],
                  ncomp=ncomp, standardize=False)
    assert_equal(_rank(p), ntotal)
    assert_equal(p['basis_vectors over t'].shape, (nimages, ntotal))
    assert_equal(p['basis_projections'].shape, data_dict['mask'].shape + (ncomp,))
    assert_equal(p['pcnt_var'].shape, (ntotal,))
    assert_almost_equal(p['pcnt_var'].sum(), 100.)
    assert_equal(p['basis_projections'].axes.coord_names, ['i','j','k','PCA components'])
    assert_equal(p['basis_projections'].coordmap.affine,
                 data_dict['fmridata'].coordmap.affine)


def test_PCANoMask():
    nimages = data_dict['nimages']
    ntotal = nimages - 1
    ncomp = 5
    p = pca_image(data_dict['fmridata'], ncomp=ncomp)
    assert_equal(_rank(p), ntotal)
    assert_equal(p['basis_vectors over t'].shape, (nimages, ntotal))
    assert_equal(p['basis_projections'].shape, data_dict['mask'].shape +
                 (ncomp,))
    assert_equal(p['pcnt_var'].shape, (ntotal,))
    assert_almost_equal(p['pcnt_var'].sum(), 100.)
    assert_equal(p['basis_projections'].axes.coord_names,
                 ['i','j','k','PCA components'])
    assert_equal(p['basis_projections'].coordmap.affine,
                 data_dict['fmridata'].coordmap.affine)


def test_PCANoMask_nostandardize():
    nimages = data_dict['nimages']
    ntotal = nimages - 1
    ncomp = 5
    p = pca_image(data_dict['fmridata'], ncomp=ncomp, standardize=False)
    assert_equal(_rank(p), ntotal)
    assert_equal(p['basis_vectors over t'].shape, (nimages, ntotal))
    assert_equal(p['basis_projections'].shape,
                 data_dict['mask'].shape + (ncomp,))
    assert_equal(p['pcnt_var'].shape, (ntotal,))
    assert_almost_equal(p['pcnt_var'].sum(), 100.)
    assert_equal(p['basis_projections'].axes.coord_names,
                 ['i','j','k','PCA components'])
    assert_equal(p['basis_projections'].coordmap.affine,
                 data_dict['fmridata'].coordmap.affine)


def test_keep():
    # Data is projected onto k=10 dimensional subspace then has its mean
    # removed. Should still have rank 10.
    k = 10
    ncomp = 5
    nimages = data_dict['nimages']
    ntotal = k
    X = np.random.standard_normal((nimages, k))
    p = pca_image(data_dict['fmridata'], ncomp=ncomp, design_keep=X)
    assert_equal(_rank(p), ntotal)
    assert_equal(p['basis_vectors over t'].shape, (nimages, ntotal))
    assert_equal(p['basis_projections'].shape,
                 data_dict['mask'].shape + (ncomp,))
    assert_equal(p['pcnt_var'].shape, (ntotal,))
    assert_almost_equal(p['pcnt_var'].sum(), 100.)
    assert_equal(p['basis_projections'].axes.coord_names,
                 ['i','j','k','PCA components'])
    assert_equal(p['basis_projections'].coordmap.affine,
                 data_dict['fmridata'].coordmap.affine)


def test_resid():
    # Data is projected onto k=10 dimensional subspace then has its mean
    # removed.  Should still have rank 10.
    k = 10
    ncomp = 5
    nimages = data_dict['nimages']
    ntotal = k
    X = np.random.standard_normal((nimages, k))
    p = pca_image(data_dict['fmridata'], ncomp=ncomp, design_resid=X)
    assert_equal(_rank(p), ntotal)
    assert_equal(p['basis_vectors over t'].shape, (nimages, ntotal))
    assert_equal(p['basis_projections'].shape,
                 data_dict['mask'].shape + (ncomp,))
    assert_equal(p['pcnt_var'].shape, (ntotal,))
    assert_almost_equal(p['pcnt_var'].sum(), 100.)
    assert_equal(p['basis_projections'].axes.coord_names,
                 ['i','j','k','PCA components'])
    assert_equal(p['basis_projections'].coordmap.affine,
                 data_dict['fmridata'].coordmap.affine)


def test_both():
    k1 = 10
    k2 = 8
    ncomp = 5
    nimages = data_dict['nimages']
    ntotal = k1
    X1 = np.random.standard_normal((nimages, k1))
    X2 = np.random.standard_normal((nimages, k2))
    p = pca_image(data_dict['fmridata'], ncomp=ncomp, design_resid=X2, design_keep=X1)

    assert_equal(_rank(p), ntotal)
    assert_equal(p['basis_vectors over t'].shape, (nimages, ntotal))
    assert_equal(p['basis_projections'].shape,
                 data_dict['mask'].shape + (ncomp,))
    assert_equal(p['pcnt_var'].shape, (ntotal,))
    assert_almost_equal(p['pcnt_var'].sum(), 100.)

    assert_equal(p['basis_projections'].axes.coord_names,
                 ['i','j','k','PCA components'])
    assert_equal(p['basis_projections'].coordmap.affine,
                 data_dict['fmridata'].coordmap.affine)


def test_5d():
    # What happened to a 5d image? We should get 4d images back
    img = data_dict['fmridata']
    data = img.get_data()
    # Make a last input and output axis called 'v'
    vcs = CS('v')
    xtra_cmap = AffineTransform(vcs, vcs, np.eye(2))
    cmap_5d = cm_product(img.coordmap, xtra_cmap)
    data_5d = data.reshape(data.shape + (1,))
    fived = Image(data_5d, cmap_5d)
    mask = data_dict['mask']
    mask_data = mask.get_data()
    mask_data = mask_data.reshape(mask_data.shape + (1,))
    cmap_4d = cm_product(mask.coordmap, xtra_cmap)
    mask4d = Image(mask_data, cmap_4d)
    nimages = data_dict['nimages']
    ntotal = nimages - 1
    ncomp = 5
    p = pca_image(fived, 't', mask4d, ncomp=ncomp)
    assert_equal(_rank(p), ntotal)
    assert_equal(p['basis_vectors over t'].shape, (nimages, ntotal))
    assert_equal(p['basis_projections'].shape, data.shape[:3] + (ncomp, 1))
    assert_equal(p['pcnt_var'].shape, (ntotal,))
    assert_almost_equal(p['pcnt_var'].sum(), 100.)

    assert_equal(p['basis_projections'].axes.coord_names,
                 ['i','j','k','PCA components','v'])
    assert_equal(p['basis_projections'].coordmap.affine,
                 fived.coordmap.affine)
    # flip the PCA dimension to end
    data_5d = data.reshape(data.shape[:3] + (1, data.shape[3]))
    # Make the last axis name be 'group'.  't' is not a length 1 dimension we
    # are going to leave as is
    gcs = CS(['group'])
    xtra_cmap = AffineTransform(gcs, gcs, np.eye(2))
    cmap_5d = cm_product(img.coordmap, xtra_cmap)
    fived = Image(data_5d, cmap_5d)
    # Give the mask a 't' dimension, but no group dimension
    mask = data_dict['mask']
    mask_data = mask.get_data()
    mask_data = mask_data.reshape(mask_data.shape + (1,))
    # We need to replicate the time scaling of the image cmap, hence the 2. in
    # the affine
    xtra_cmap = AffineTransform(CS('t'), CS('t'), np.diag([2., 1]))
    cmap_4d = cm_product(mask.coordmap, xtra_cmap)
    mask4d = Image(mask_data, cmap_4d)
    nimages = data_dict['nimages']
    ntotal = nimages - 1
    ncomp = 5
    # We can now show the axis does not have to be time
    p = pca_image(fived, mask=mask4d, ncomp=ncomp, axis='group')
    assert_equal(p['basis_vectors over group'].shape, (nimages, ntotal))
    assert_equal(p['basis_projections'].axes.coord_names,
                 ['i','j','k','t','PCA components'])
    assert_equal(p['basis_projections'].shape,
                 data.shape[:3] + (1, ncomp))


def img_res2pos1(res, bv_key):
    # Orient basis vectors in standard direction
    axis = res['axis']
    bvs = res[bv_key]
    bps_img = res['basis_projections']
    bps = bps_img.get_data()
    signs = np.sign(bvs[0])
    res[bv_key] = bvs * signs
    new_axes = [None] * bps.ndim
    n_comps = bps.shape[axis]
    new_axes[axis] = slice(0, n_comps)
    res['basis_projections'] = Image(bps * signs[new_axes],
                                     bps_img.coordmap)
    return res


def test_other_axes():
    # With a diagonal affine, we can do PCA on any axis
    ncomp = 5
    img = data_dict['fmridata']
    in_coords = list(img.axes.coord_names)
    img_data = img.get_data()
    for axis_no, axis_name in enumerate('ijkt'):
        p = pca_image(img, axis_name, ncomp=ncomp)
        n = img.shape[axis_no]
        bv_key = 'basis_vectors over ' + axis_name
        assert_equal(_rank(p), n - 1)
        assert_equal(p[bv_key].shape, (n, n - 1))
        # We get the expected data back
        dp = pca_array(img_data, axis_no, ncomp=ncomp)
        # We have to make sure the signs are the same; on Windows it seems the
        # signs can flip even between two runs on the same data
        pos_p = img_res2pos1(p, bv_key)
        pos_dp = res2pos1(dp)
        img_bps = pos_p['basis_projections']
        assert_almost_equal(pos_dp['basis_vectors'], pos_p[bv_key])
        assert_almost_equal(pos_dp['basis_projections'], img_bps.get_data())
        # And we've replaced the expected axis
        exp_coords = in_coords[:]
        exp_coords[exp_coords.index(axis_name)] = 'PCA components'
        assert_equal(img_bps.axes.coord_names, exp_coords)
    # If the affine is not diagonal, we'll get an error
    aff = from_matvec(np.arange(16).reshape(4,4))
    nd_cmap = AffineTransform(img.axes, img.reference, aff)
    nd_img = Image(img_data, nd_cmap)
    for axis_name in 'ijkt':
        assert_raises(AxisError, pca_image, nd_img, axis_name)
    # Only for the non-diagonal parts
    aff = np.array([[1, 2, 0, 0, 10],
                    [2, 1, 0, 0, 11],
                    [0, 0, 3, 0, 12],
                    [0, 0, 0, 4, 13],
                    [0, 0, 0, 0, 1]])
    nd_cmap = AffineTransform(img.axes, img.reference, aff)
    nd_img = Image(img_data, nd_cmap)
    for axis_name in 'ij':
        assert_raises(AxisError, pca_image, nd_img, axis_name)
    for axis_name in 'kt':
        p = pca_image(img, axis_name, ncomp=ncomp)
        exp_coords = in_coords[:]
        exp_coords[exp_coords.index(axis_name)] = 'PCA components'
        assert_equal(p['basis_projections'].axes.coord_names, exp_coords)



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import nipy.algorithms.statistics.tests.test_rft
import nipy.algorithms.statistics.tests.test_intrinsic_volumes



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Test the discrete_domain utilities.

Caveat assumes that the MNI template image is available at
in ~/.nipy/tests/data
"""

import numpy as np
from numpy.testing import assert_almost_equal, assert_equal
from ..discrete_domain import smatrix_from_nd_idx, smatrix_from_3d_array, \
    smatrix_from_nd_array, domain_from_binary_array, domain_from_image, \
    domain_from_mesh, grid_domain_from_binary_array, grid_domain_from_image, \
    grid_domain_from_shape
from nibabel import Nifti1Image
import nibabel.gifti as nbg

from nipy.testing.decorators import skipif

GOOD_GIFTI = hasattr(nbg, 'GiftiDataArray')

shape = np.array([5, 6, 7, 8, 9])


def generate_dataset(shape):
    """Generate a dataset with the described shape
    """
    dim = len(shape)
    idx = np.reshape(np.indices(shape), (dim, -1)).T
    return idx


def test_smatrix_1d():
    """Test the 1-d topological domain
    """
    idx = generate_dataset(shape[:1])
    sm = smatrix_from_nd_idx(idx, nn=0)
    assert_equal(sm.data.size, 2 * shape[0] - 2)


def test_smatrix_2d():
    """Test the 2-d topological domain
    """
    idx = generate_dataset(shape[:2])
    sm = smatrix_from_nd_idx(idx, nn=0)
    ne = 2 * (2 * np.prod(shape[:2]) - shape[0] - shape[1])
    assert_equal(sm.data.size, ne)


def test_smatrix_3d():
    """Test the 3-d topological domain
    """
    idx = generate_dataset(shape[:3])
    sm = smatrix_from_nd_idx(idx)
    ne = 2 * (3 * np.prod(shape[:3]) - shape[0] * shape[1]
              - shape[0] * shape[2] - shape[1] * shape[2])
    assert_equal(sm.data.size, ne)


def test_smatrix_4d():
    """Test the 4-d topological domain
    """
    idx = generate_dataset(shape[:4])
    sm = smatrix_from_nd_idx(idx)
    ne = 4 * np.prod(shape[:4])
    for d in range(4):
        ne -= np.prod(shape[:4]) / shape[d]
    ne *= 2
    assert_equal(sm.data.size, ne)


def test_smatrix_5d():
    """Test the 5-d topological domain
    """
    idx = generate_dataset(shape)
    sm = smatrix_from_nd_idx(idx)
    ne = 5 * np.prod(shape)
    for d in range(5):
        ne -= np.prod(shape) / shape[d]
    ne *= 2
    assert_equal(sm.data.size, ne)


def test_smatrix_5d_bis():
    """Test the 5-d topological domain
    """
    toto = np.ones(shape)
    sm = smatrix_from_nd_array(toto)
    ne = 5 * np.prod(shape)
    for d in range(5):
        ne -= np.prod(shape) / shape[d]
    ne *= 2
    assert_equal(sm.data.size, ne)


def test_matrix_from_3d_array():
    """Test the topology using the nipy.graph approach
    """
    toto = np.ones(shape[:3])
    sm = smatrix_from_3d_array(toto, 6)
    ne = 3 * np.prod(shape[:3])
    for d in range(3):
        ne -= np.prod(shape[:3]) / shape[d]
    ne *= 2
    print sm.data, ne
    assert_equal((sm.data > 0).sum(), ne)


def test_array_domain():
    """Test the construction of domain based on array
    """
    toto = np.ones(shape)
    ddom = domain_from_binary_array(toto)
    assert_equal(np.sum(ddom.local_volume), np.prod(shape))


def test_connected_components():
    """Test the estimation of connected components
    """
    toto = np.ones(shape)
    ddom = domain_from_binary_array(toto)
    assert_equal(ddom.connected_components(), np.zeros(ddom.size))


def test_image_domain():
    """Test the construction of domain based on image
    """
    toto = np.ones(shape[:3])
    affine = np.random.randn(4, 4)
    affine[3:, 0:3] = 0
    nim = Nifti1Image(toto, affine)
    ddom = domain_from_image(nim)
    ref = np.sum(toto) * np.absolute(np.linalg.det(affine))
    assert_almost_equal(np.sum(ddom.local_volume), ref)


def test_image_feature():
    """Test the construction of domain based on image and related feature
    """
    mask = np.random.randn(*shape[:3]) > .5
    noise = np.random.randn(*shape[:3])
    affine = np.eye(4)
    mim = Nifti1Image(mask.astype('u8'), affine)
    nim = Nifti1Image(noise, affine)
    ddom = grid_domain_from_image(mim)
    ddom.make_feature_from_image(nim, 'noise')
    assert_almost_equal(ddom.features['noise'], noise[mask])


def test_array_grid_domain():
    """Test the construction of grid domain based on array
    """
    toto = np.ones(shape)
    ddom = grid_domain_from_binary_array(toto)
    assert_equal(np.sum(ddom.local_volume), np.prod(shape))


def test_image_grid_domain():
    """Test the construction of grid domain based on image
    """
    toto = np.ones(shape[:3])
    affine = np.random.randn(4, 4)
    affine[3:, 0:3] = 0
    nim = Nifti1Image(toto, affine)
    ddom = grid_domain_from_image(nim)
    ref = np.sum(toto) * np.absolute(np.linalg.det(affine[:3, 0:3]))
    assert_almost_equal(np.sum(ddom.local_volume), ref)


def test_shape_grid_domain():
    """
    """
    ddom = grid_domain_from_shape(shape)
    assert_equal(np.sum(ddom.local_volume), np.prod(shape))


def test_feature():
    """ test feature inclusion
    """
    toto = np.random.rand(*shape)
    ddom = domain_from_binary_array(toto)
    ddom.set_feature('data', np.ravel(toto))
    plop = ddom.get_feature('data')
    assert_almost_equal(plop, np.ravel(toto))


def test_mask_feature():
    """ test_feature_masking
    """
    toto = np.random.rand(*shape)
    ddom = domain_from_binary_array(toto)
    ddom.set_feature('data', np.ravel(toto))
    mdom = ddom.mask(np.ravel(toto > .5))
    plop = mdom.get_feature('data')
    assert_almost_equal(plop, toto[toto > .5])


def test_domain_mask():
    """test domain masking
    """
    toto = np.random.rand(*shape)
    ddom = domain_from_binary_array(toto)
    mdom = ddom.mask(np.ravel(toto > .5))
    assert_equal(mdom.size, np.sum(toto > .5))


def test_grid_domain_mask():
    """test grid domain masking
    """
    toto = np.random.rand(*shape)
    ddom = grid_domain_from_binary_array(toto)
    mdom = ddom.mask(np.ravel(toto > .5))
    assert_equal(mdom.size, np.sum(toto > .5))


@skipif(not GOOD_GIFTI)
def test_domain_from_mesh():
    """Test domain_from_mesh method
    """
    coords = np.array([[0., 0., 0.],
                       [0., 0., 1.],
                       [0., 1., 0.],
                       [1., 0., 0.]])
    triangles = np.asarray([[0, 1, 2],
                            [0, 1, 3],
                            [0, 2, 3],
                            [1, 2, 3]])
    darrays = [nbg.GiftiDataArray(coords)] + [nbg.GiftiDataArray(triangles)]
    toy_image = nbg.GiftiImage(darrays=darrays)
    domain = domain_from_mesh(toy_image)
    # if we get there, we could build the domain, and that's what we wanted.
    assert_equal(domain.get_coord(), coords)


def test_representative():
    """ test representative computation
    """
    toto = np.random.rand(*shape)
    ddom = domain_from_binary_array(toto)
    ddom.set_feature('data', np.ravel(toto))
    dmean = toto.mean()
    dmin = toto.min()
    dmax = toto.max()
    dmed = np.median(toto)
    assert_almost_equal(ddom.representative_feature('data', 'mean'), dmean)
    assert_almost_equal(ddom.representative_feature('data', 'min'), dmin)
    assert_almost_equal(ddom.representative_feature('data', 'max'), dmax)
    assert_almost_equal(ddom.representative_feature('data', 'median'), dmed)


def test_integrate_1d():
    """ test integration in 1d
    """
    toto = np.random.rand(*shape)
    ddom = domain_from_binary_array(toto)
    ddom.set_feature('data', np.ravel(toto))
    assert_almost_equal(ddom.integrate('data'), toto.sum())


def test_integrate_2d():
    """test integration in 2d
    """
    toto = np.random.rand(*shape)
    ddom = domain_from_binary_array(toto)
    ftoto = np.ravel(toto)
    f2 = np.vstack((ftoto, ftoto)).T
    ddom.set_feature('data', f2)
    ts = np.ones(2) * toto.sum()
    assert_almost_equal(ddom.integrate('data'), ts)


if __name__ == "__main__":
    import nose
    nose.run(argv=['', __file__])



""" Routines for Python 3 compatibility

These are in addition to the nibabel.py3k routines.
"""

import sys
py3 = sys.version_info[0] >= 3

if py3:
    def to_str(s):
        """ Convert `s` to string, decoding as latin1 if `s` is bytes
        """
        if isinstance(s, bytes):
            return s.decode('latin1')
        return str(s)
else:
    to_str = str


def open4csv(fname, mode):
    """ Open filename `fname` for CSV IO in read or write `mode`

    Parameters
    ----------
    fname : str
        filename to open
    mode : {'r', 'w'}
        Mode to open file.  Don't specify binary or text modes; we need to
        chose these according to python version.

    Returns
    -------
    fobj : file object
        open file object; needs to be closed by the caller
    """
    if mode not in ('r', 'w'):
        raise ValueError('Only "r" and "w" allowed for mode')
    if not py3: # Files for csv reading and writing should be binary mode
        return open(fname, mode + 'b')
    return open(fname, mode, newline='')



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
''' Diagnostic 4d image screen '''
from os.path import join as pjoin

import numpy as np

from ...core.api import Image, drop_io_dim, append_io_dim
from ...io.api import save_image
from ..utils import pca
from .timediff import time_slice_diffs
from .tsdiffplot import plot_tsdiffs


def screen(img4d, ncomp=10):
    ''' Diagnostic screen for 4d FMRI image

    Includes PCA, tsdiffana and mean, std, min, max images.

    Parameters
    ----------
    img4d : ``Image``
       4d image file
    ncomp : int, optional
       number of component images to return.  Default is 10

    Returns
    -------
    screen : dict
       with keys:

       * mean : mean image (all summaries are over last dimension)
       * std : standard deviation image
       * max : image of max
       * min : min
       * pca : 4D image of PCA component images
       * pca_res : dict of results from PCA
       * ts_res : dict of results from tsdiffana

    Examples
    --------
    >>> import nipy as ni
    >>> from nipy.testing import funcfile
    >>> img = ni.load_image(funcfile)
    >>> screen_res = screen(img)
    >>> screen_res['mean'].ndim
    3
    >>> screen_res['pca'].ndim
    4
    '''
    if img4d.ndim != 4:
        raise ValueError('Expecting a 4d image')
    data = img4d.get_data()
    cmap = img4d.coordmap
    cmap_3d = drop_io_dim(cmap, 't')
    screen_res = {}
    # standard processed images
    screen_res['mean'] = Image(np.mean(data, axis=-1), cmap_3d)
    screen_res['std'] = Image(np.std(data, axis=-1), cmap_3d)
    screen_res['max'] = Image(np.max(data, axis=-1), cmap_3d)
    screen_res['min'] = Image(np.min(data, axis=-1), cmap_3d)
    # PCA
    screen_res['pca_res'] = pca.pca(data,
                                    axis=-1,
                                    standardize=False,
                                    ncomp=ncomp)
    cmap_4d = append_io_dim(cmap_3d, 'l' , 't')
    screen_res['pca'] = Image(screen_res['pca_res']['basis_projections'],
                              cmap_4d)
    # tsdiffana
    screen_res['ts_res'] = time_slice_diffs(data)
    return screen_res


def write_screen_res(res, out_path, out_root,
                     out_img_ext='.nii',
                     pcnt_var_thresh=0.1):
    ''' Write results from ``screen`` to disk as images

    Parameters
    ----------
    res : dict
       output from ``screen`` function
    out_path : str
       directory to which to write output images
    out_root : str
       part of filename between image-specific prefix and image-specific
       extension to use for writing images
    out_img_ext : str, optional
       extension (identifying image type) to which to write volume
       images.  Default is '.nii'
    pcnt_var_thresh : float, optional
       threshold below which we do not plot percent variance explained
       by components; default is 0.1.  This removes the long tail from
       percent variance plots.

    Returns
    -------
    None
    '''
    import matplotlib.pyplot as plt
    # save volume images
    for key in ('mean', 'min', 'max', 'std', 'pca'):
        fname = pjoin(out_path, '%s_%s%s' % (key,
                                             out_root,
                                             out_img_ext))
        save_image(res[key], fname)
    # plot, save component time courses and some tsdiffana stuff
    ncomp = res['pca'].shape[-1]
    vectors = res['pca_res']['basis_vectors']
    pcnt_var = res['pca_res']['pcnt_var']
    np.savez(pjoin(out_path, 'vectors_components_%s.npz' % out_root),
             basis_vectors=vectors,
             pcnt_var=pcnt_var,
             volume_means=res['ts_res']['volume_means'],
             slice_mean_diff2=res['ts_res']['slice_mean_diff2'],
            )
    plt.figure()
    for c in range(ncomp):
        plt.subplot(ncomp, 1, c+1)
        plt.plot(vectors[:,c])
        plt.axis('tight')
    plt.suptitle(out_root + ': PCA basis vectors')
    plt.savefig(pjoin(out_path, 'components_%s.png' % out_root))
    # plot percent variance
    plt.figure()
    plt.plot(pcnt_var[pcnt_var >= pcnt_var_thresh])
    plt.axis('tight')
    plt.suptitle(out_root + ': PCA percent variance')
    plt.savefig(pjoin(out_path, 'pcnt_var_%s.png' % out_root))    
    # plot tsdiffana
    plt.figure()
    axes = [plt.subplot(4, 1, i+1) for i in range(4)]
    plot_tsdiffs(res['ts_res'], axes)
    plt.suptitle(out_root + ': tsdiffana')
    plt.savefig(pjoin(out_path, 'tsdiff_%s.png' % out_root))



#!/usr/bin/env python
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import print_function
__doc__ = """
Example where the result of the min of two contrasts is computed and displayed.
This is based on the Localizer dataset, in which we want to find the regions
activated both in left and right finger tapping.

Notes
-----
This is the valid conjunction test discussed in:
Nichols T, Brett M, Andersson J, Wager T, Poline JB. Valid conjunction
inference with the minimum statistic. Neuroimage. 2005 Apr 15;25(3):653-60.

Needs matplotlib

Author : Bertrand Thirion, 2012
"""
print(__doc__)

from os import mkdir, getcwd, path

import numpy as np

try:
    import matplotlib.pyplot as plt
except ImportError:
    raise RuntimeError("This script needs the matplotlib library")

from nibabel import save

from nipy.modalities.fmri.glm import FMRILinearModel
from nipy.modalities.fmri.design_matrix import make_dmtx
from nipy.modalities.fmri.experimental_paradigm import \
    load_paradigm_from_csv_file
from nipy.labs.viz import plot_map, cm

# Local import
from get_data_light import DATA_DIR, get_first_level_dataset

#######################################
# Data and analysis parameters
#######################################

# volume mask
# This dataset is large
get_first_level_dataset()
data_path = path.join(DATA_DIR, 's12069_swaloc1_corr.nii.gz')
paradigm_file = path.join(DATA_DIR, 'localizer_paradigm.csv')

# timing
n_scans = 128
tr = 2.4

# paradigm
frametimes = np.linspace(0, (n_scans - 1) * tr, n_scans)

# confounds
hrf_model = 'canonical'
drift_model = 'cosine'
hfcut = 128

# write directory
write_dir = path.join(getcwd(), 'results')
if not path.exists(write_dir):
    mkdir(write_dir)

print('Computation will be performed in directory: %s' % write_dir)

########################################
# Design matrix
########################################

print('Loading design matrix...')

paradigm = load_paradigm_from_csv_file(paradigm_file)['0']

design_matrix = make_dmtx(frametimes, paradigm, hrf_model=hrf_model,
                          drift_model=drift_model, hfcut=hfcut)

#########################################
# Specify the contrasts
#########################################

# simplest ones
contrasts = {}
n_columns = len(design_matrix.names)
for i in range(paradigm.n_conditions):
    contrasts['%s' % design_matrix.names[i]] = np.eye(n_columns)[i]

# and more complex/ interesting ones
contrasts['left'] = contrasts['clicGaudio'] + contrasts['clicGvideo']
contrasts['right'] = contrasts['clicDaudio'] + contrasts['clicDvideo']

########################################
# Perform a GLM analysis
########################################

print('Fitting a General Linear Model')
fmri_glm = FMRILinearModel(data_path, design_matrix.matrix,
                           mask='compute')
fmri_glm.fit(do_scaling=True, model='ar1')

#########################################
# Estimate the contrasts
#########################################

contrast_id = 'left_right_motor_min'
z_map, effects_map = fmri_glm.contrast(
    np.vstack((contrasts['left'], contrasts['right'])), 
    contrast_type='tmin-conjunction', output_z=True, output_effects=True)
z_image_path = path.join(write_dir, '%s_z_map.nii' % contrast_id)
save(z_map, z_image_path)

contrast_path = path.join(write_dir, '%s_con.nii' % contrast_id)
save(effects_map, contrast_path)
# note that the effects_map is two-dimensional: 
# these dimensions correspond to 'left' and 'right'

# Create snapshots of the contrasts
vmax = max(- z_map.get_data().min(), z_map.get_data().max())
plot_map(z_map.get_data(), fmri_glm.affine,
         cmap=cm.cold_hot,
         vmin=- vmax,
         vmax=vmax,
         anat=None,
         figure=10,
         threshold=2.5)
plt.savefig(path.join(write_dir, '%s_z_map.png' % contrast_id))
plt.show()

print('All the  results were witten in %s' % write_dir)
# Note: fancier visualization of the results are shown
# in the viz3d example



#!/usr/bin/env python

import numpy as np
import numpy.random as nr
from unittest import TestCase

from ..graph import (WeightedGraph, complete_graph, mst, knn, eps_nn, 
                     wgraph_from_adjacency, wgraph_from_coo_matrix, 
                     concatenate_graphs, wgraph_from_3d_grid)


def basicdata():
    x = np.array( [[-1.998,-2.024], [-0.117,-1.010], [1.099,-0.057],
                   [ 1.729,-0.252], [1.003,-0.021], [1.703,-0.739],
                   [-0.557,1.382],[-1.200,-0.446],[-0.331,-0.256],
                   [-0.800,-1.584]])
    return x

def basic_graph():
    l = np.linspace(0, 2 * np.pi, 20, endpoint=False)
    x = np.column_stack((np.cos(l), np.sin(l)))
    G = knn(x, 2)
    return G

def basic_graph_2():
    l = np.linspace(0, 2 * np.pi, 20, endpoint=False)
    x = np.column_stack((np.cos(l), np.sin(l)))
    G = knn(x, 2)
    return G, x

class test_Graph(TestCase):
    
    def test_complete(self):
        v = 10
        G = complete_graph(v)
        a = G.get_edges()[:, 0]
        b = G.get_edges()[:, 1]
        inds = np.indices((v, v)).reshape( (2, v * v) )
        self.assert_( ( inds == (a, b) ).all() )
  
    def test_knn_1(self):
        x = basicdata()
        G = knn(x, 1)
        A = G.get_edges()[:,0]
        OK = (np.shape(A)[0] == (14))
        self.assert_(OK)
    
    def test_set_euclidian(self):
        G,x = basic_graph_2()
        d = G.weights
        G.set_euclidian(x / 10)
        D = G.weights
        OK = np.allclose(D, d / 10, 1e-7)
        self.assert_(OK)

    def test_set_gaussian(self):
        G,x = basic_graph_2()
        d = G.weights
        G.set_gaussian(x, 1.0)
        D = G.weights
        OK = np.allclose(D, np.exp(- d * d / 2), 1e-7)
        self.assert_(OK)

    def test_set_gaussian_2(self):
        G,x = basic_graph_2()
        d = G.weights
        G.set_gaussian(x)
        D = G.weights
        sigma = sum(d * d) / len(d)
        OK = np.allclose(D, np.exp(-d * d / (2 * sigma)), 1e-7)
        self.assert_(OK)

    def test_eps_1(self):
        x = basicdata()
        G = eps_nn(x, 1.)
        D = G.weights
        OK = (np.size(D) == 16)
        self.assert_(OK)
        OK = (D < 1).all()
        self.assert_(OK)

    def test_mst_1(self):
        x = basicdata()
        G = mst(x)
        D = G.weights
        OK = (np.size(D) == 18)
        self.assert_(OK)

    def test_3d_grid(self):
        """test the 6nn graph
        """
        x0 = np.array([0, 0, 0])
        x1 = np.array([[1, 0, 0], [0, 1, 0], [0, 0, 1], [-1, 0, 0], [0, -1, 0],
                      [0, 0, -1]])
        x2 = np.array([[1, 1, 0], [0, 1, 1], [1, 0, 1], [1, -1, 0], [0, 1, -1],
                      [1, 0, -1], [-1, 1, 0], [0, -1, 1], [-1, 0, 1], 
                      [-1, -1, 0], [-1, 0, -1], [0, -1, -1]])
        x3 = np.array([[1, 1, 1], [1, 1, -1], [1, -1, 1], [1, -1, -1],
                       [-1, 1, 1], [-1, 1, -1], [-1, -1, 1], [-1, -1, -1]])
        for x in x1:
            xyz = np.vstack((x0, x))
            assert wgraph_from_3d_grid(xyz, 6).E == 2
            assert wgraph_from_3d_grid(xyz, 18).E == 2
            assert wgraph_from_3d_grid(xyz, 26).E == 2
        for x in x2:
            xyz = np.vstack((x0, x))
            assert wgraph_from_3d_grid(xyz, 6).E == 0
            assert wgraph_from_3d_grid(xyz, 18).E == 2
            assert wgraph_from_3d_grid(xyz, 26).E == 2
        for x in x3:
            xyz = np.vstack((x0, x))
            assert wgraph_from_3d_grid(xyz, 6).E == 0
            assert wgraph_from_3d_grid(xyz, 18).E == 0
            assert wgraph_from_3d_grid(xyz, 26).E == 2
            

    def test_grid_3d_1(self):
        """ Test the 6 nn graphs on 3d grid
        """
        nx, ny, nz = 9, 6, 1
        xyz = np.mgrid[0:nx, 0:ny, 0:nz]
        xyz = np.reshape(xyz, (3, nx * ny * nz)).T
        G = wgraph_from_3d_grid(xyz, 6)
        self.assert_(G.E == 186)
    
    def test_grid_3d_2(self):
        """ Test the 18-nn graph on a 3d grid
        """
        nx, ny, nz = 9, 6, 1
        xyz = np.mgrid[0:nx, 0:ny, 0:nz]
        xyz = np.reshape(xyz,(3, nx * ny * nz)).T
        G = wgraph_from_3d_grid(xyz, 18)
        self.assert_(G.E == 346)
        
    def test_grid_3d_3(self):
        """ Test the 26-nn graph on a 3d grid
        """
        nx, ny, nz = 9, 6, 1
        xyz = np.mgrid[0:nx, 0:ny, 0:nz]
        xyz = np.reshape(xyz,(3, nx * ny * nz)).T
        G = wgraph_from_3d_grid(xyz, 26)
        self.assert_(G.E == 346)

    def test_grid_3d_4(self):
        nx, ny, nz = 10, 10, 10
        xyz = np.reshape(np.indices((nx, ny, nz)), (3, nx * ny * nz)).T
        G = wgraph_from_3d_grid(xyz, 26)
        D = G.weights
        # 6 * 9 * 10 * 10
        self.assert_(sum(D == 1)==5400 )
        # 26 * 8 ** 3 + 6 * 8 ** 2 * 17 + 12 * 8 * 11 + 8 * 7 
        self.assert_(np.size(D) == 20952 )
        # 18 * 8 ** 3 + 6 * 8 ** 2 * 13 + 12 * 8 * 9 + 8 * 6
        self.assert_(sum(D < 1.5) == 15120)

    def test_grid_3d_5(self):
        nx, ny, nz = 5, 5, 5
        xyz = np.reshape(np.indices((nx, ny, nz)), (3, nx * ny * nz)).T
        G = wgraph_from_3d_grid(xyz, 26)
        D = G.weights.copy()
        G.set_euclidian(xyz)
        assert (np.allclose(G.weights, D, 1.e-7))

    def test_grid_3d_6(self):
        nx, ny, nz = 5, 5, 5
        xyz = np.reshape(np.indices((nx, ny, nz)), (3, nx * ny * nz)).T
        adj = wgraph_from_3d_grid(xyz, 26).to_coo_matrix().tolil()
        assert len(adj.rows[63]) == 26
        for i in [62, 64, 58, 68, 38, 88, 57, 67, 37, 87, 59, 69, 39, 89, 33, 
                  83, 43, 93, 32, 82, 42, 92, 34, 84, 44, 94]:
            assert i in adj.rows[63]

    def test_grid_3d_7(self):
        """ Check that the grid graph is symmetric
        """
        xyz = np.array(np.where(np.random.rand(5, 5, 5) > 0.5)).T
        adj = wgraph_from_3d_grid(xyz, 6).to_coo_matrix()
        assert (adj - adj.T).nnz == 0
        adj = wgraph_from_3d_grid(xyz, 18).to_coo_matrix()
        assert (adj - adj.T).nnz == 0
        adj = wgraph_from_3d_grid(xyz, 26).to_coo_matrix()
        assert (adj - adj.T).nnz == 0
        

    def test_cut_redundancies(self):
        G = basic_graph()
        e = G.E
        edges = G.get_edges()
        weights = G.weights
        G.E = 2 * G.E
        G.edges = np.concatenate((edges, edges))
        G.weights = np.concatenate((weights, weights))
        K = G.cut_redundancies()
        OK = (K.E == e)
        self.assert_(OK)

    def test_degrees(self):
        G = basic_graph()
        (r,l) = G.degrees()
        self.assert_(( r == 2 ).all())
        self.assert_(( l == 2 ).all())

    def test_normalize(self):
        G = basic_graph()
        G.normalize()
        M = G.to_coo_matrix()
        sM = np.array(M.sum(1)).ravel()
        test = np.absolute(sM - 1) < 1.e-7
        OK = np.size(np.nonzero(test) == 0)
        self.assert_(OK)

    def test_normalize_2(self):
        G = basic_graph()
        G.normalize(0)
        M = G.to_coo_matrix()
        sM = np.array(M.sum(1)).ravel()
        test = np.absolute(sM - 1) < 1.e-7
        OK = np.size(np.nonzero(test)==0)
        self.assert_(OK)

    def test_normalize_3(self):
        G = basic_graph()
        G.normalize(1)
        M = G.to_coo_matrix()
        sM = np.array(M.sum(0)).ravel()
        test = np.absolute(sM - 1) < 1.e-7
        OK = np.size(np.nonzero(test)==0)
        self.assert_(OK)

    def test_adjacency(self):
        G = basic_graph()
        M = G.to_coo_matrix()
        self.assert_(( M.diagonal() == 0 ).all())
        A = M.toarray()
        self.assert_(( np.diag(A, 1) != 0 ).all())
        self.assert_(( np.diag(A, -1) != 0 ).all())       

    def test_cc(self):
        G = basic_graph()
        l = G.cc()
        L = np.array(l==0)
        OK = L.all()
        self.assert_(OK)

    def test_isconnected(self):
        G = basic_graph()
        self.assert_(G.is_connected())

    def test_main_cc(self):
        x = basicdata()
        G = knn(x, 1)
        l = G.cc()
        l = G.main_cc()
        assert np.size(l)==6

    def test_dijkstra(self):
        """ Test dijkstra's algorithm
        """
        G = basic_graph()
        l = G.dijkstra(0)
        assert (np.absolute(l[10] - 20 * np.sin(np.pi / 20)) < 1.e-7)

    def test_dijkstra_multiseed(self):
        """ Test dijkstra's algorithm, multi_seed version
        """
        G = basic_graph()
        l = G.dijkstra([0, 1])
        assert (np.absolute(l[10] - 18 * np.sin(np.pi / 20)) < 1.e-7)


    def test_dijkstra2(self):
        """ Test dijkstra's algorithm, API detail
        """
        G = basic_graph()
        l = G.dijkstra()
        assert (np.absolute(l[10] - 20 * np.sin(np.pi / 20)) < 1.e-7)
        
    def test_compact_representation(self):
        """ Test that the compact representation of the graph is indeed correct
        """
        G = basic_graph()
        idx, ne, we = G.compact_neighb()
        assert len(idx) == 21
        assert idx[0] == 0
        assert idx[20] == G.E
        assert len(ne) == G.E
        assert len(we) == G.E

    def test_floyd_1(self):
        """ Test Floyd's algo without seed
        """
        G = basic_graph()
        l = G.floyd()
        for i in range(10):
            plop = np.absolute(np.diag(l, i) - 2 * i * np.sin(2 * np.pi / 40))
            assert(plop.max()<1.e-4)

    def test_floyd_2(self):
        """ Test Floyd's algo, with seed
        """
        G = basic_graph()
        seeds = np.array([0,10])
        l = G.floyd(seeds)
        
        for i in range(10):
            plop = np.absolute(l[0,i]-2*i*np.sin(2*np.pi/40))
            assert (plop.max()<1.e-4)
            plop = np.absolute(l[0,19-i]-2*(i+1)*np.sin(2*np.pi/40))
            assert (plop.max()<1.e-4)

        for i in range(10):
            plop = np.absolute(l[1,i]-2*(10-i)*np.sin(2*np.pi/40))
            assert (plop.max()<1.e-4)
            plop = np.absolute(l[1,19-i]-2*(9-i)*np.sin(2*np.pi/40))
            assert (plop.max()<1.e-4)
  
    def test_symmeterize(self):
        a = np.array([0, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6])
        b = np.array([1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 0, 0, 1])
        edges = np.vstack((a, b)).T
        d = np.ones(14)
        G = WeightedGraph(7, edges, d)
        G.symmeterize()
        d = G.weights
        ok = (d == 0.5)
        self.assert_(ok.all())

    def test_voronoi(self):
        """ test voronoi labelling with 2 seeds
        """
        a = np.array([0, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6])
        b = np.array([1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 0, 0, 1])
        d = np.array([1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2]);
        edges = np.transpose(np.vstack((a, b)))
        G = WeightedGraph(7, edges,d)
        G.symmeterize()
        seed = np.array([0, 6])
        label = G.voronoi_labelling(seed)
        assert(label[1] == 0)
        
    def test_voronoi2(self):
        """ test voronoi labelling with one seed
        """
        a = np.array([0, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6])
        b = np.array([1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 0, 0, 1])
        d = np.array([1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2, 1, 2]);
        edges = np.vstack((a, b)).T
        G = WeightedGraph(7, edges,d)
        G.symmeterize()
        seed = np.array([0])
        label = G.voronoi_labelling(seed)
        assert(label[4] == 0)
 
    def test_voronoi3(self):
        """ test voronoi labelling with non-connected components
        """
        a = np.array([0, 1, 2, 5, 6])
        b = np.array([1, 2, 3, 6, 0])
        d = np.array([1, 1, 1, 1, 1]);
        edges = np.vstack((a, b)).T
        G = WeightedGraph(7, edges,d)
        G.symmeterize()
        seed = np.array([0])
        label = G.voronoi_labelling(seed)
        assert(label[4] == - 1)

    def test_concatenate1(self,n=10,verbose=0):
        x1 = nr.randn(n,2) 
        x2 = nr.randn(n,2) 
        G1 = knn(x1, 5)
        G2 = knn(x2, 5) 
        G = concatenate_graphs(G1, G2)
        if verbose:
            G.plot(np.hstack((x1, x2)))
        self.assert_(G.cc().max()>0)

    def test_concatenate2(self,n=10,verbose=0):
        G1 = complete_graph(n)
        G2 = complete_graph(n)
        G = concatenate_graphs(G1, G2)
        self.assert_(G.cc().max() == 1)

    def test_anti_symmeterize(self,verbose=0):
        n = 10
        eps = 1.e-7
        M = (nr.rand(n, n) > 0.7).astype(np.float) 
        C = M - M.T
        G = wgraph_from_adjacency(M)
        G.anti_symmeterize()
        A = G.to_coo_matrix()
        self.assert_(np.sum(C - A) ** 2 < eps)

    def test_subgraph_1(self,n=10,verbose=0):
        x = nr.randn(n, 2) 
        G = WeightedGraph(x.shape[0])
        valid = np.zeros(n)
        g = G.subgraph(valid)
        self.assert_(g is None)

    def test_subgraph_2(self,n=10,verbose=0):
        x = nr.randn(n,2) 
        G = knn(x, 5)
        valid = np.zeros(n)
        valid[:n/2] = 1
        g = G.subgraph(valid)
        self.assert_(g.edges.max() < n / 2)

    def test_graph_create_from_array(self):
        """
        Test the creation of a graph from a sparse coo_matrix 
        """
        a = np.random.randn(5, 5)
        wg = wgraph_from_adjacency(a)
        b = wg.to_coo_matrix()
        self.assert_((a == b.todense()).all())
        
    def test_graph_create_from_coo_matrix(self):
        """
        Test the creation of a graph from a sparse coo_matrix 
        """
        import scipy.sparse as spp
        a = (np.random.randn(5, 5) > .8).astype(np.float)
        s = spp.coo_matrix(a)
        wg = wgraph_from_coo_matrix(s)
        b = wg.to_coo_matrix()
        self.assert_((b.todense() == a).all())

    def test_to_coo_matrix(self):
        """ Test the generation of a sparse matrix as output 
        """
        a = (np.random.randn(5, 5)>.8).astype(np.float)
        wg = wgraph_from_adjacency(a)
        b = wg.to_coo_matrix().todense()
        self.assert_((a==b).all())
    
    def test_list_neighbours(self):
        """ test the generation of neighbours list
        """
        bg = basic_graph()
        nl = bg.list_of_neighbors()
        assert(len(nl) == bg.V)
        for ni in nl:
            assert len(ni)== 2
    
    def test_kruskal(self):
        """ test Kruskal's algor to thin the graph
        """
        x = basicdata()
        dmax = np.sqrt((x ** 2).sum())
        m = mst(x)
        g = eps_nn(x, dmax)
        k = g.kruskal()
        assert np.abs(k.weights.sum() - m.weights.sum() < 1.e-7)

    def test_concatenate3(self):
        """ test the graph concatenation utlitity
        """
        bg = basic_graph()
        cg = concatenate_graphs(bg, bg)
        valid = np.zeros(cg.V)
        valid[:bg.V] = 1
        sg = cg.subgraph(valid)
        assert (sg.edges == bg.edges).all()
        assert (sg.weights == bg.weights).all()

    def test_cliques(self):
        """ test the computation of cliques
        """
        x = np.random.rand(20, 2)
        x[15:] += 2.
        g = knn(x, 5)
        g.set_gaussian(x, 1.)
        cliques = g.cliques()
        assert len(np.unique(cliques)) > 1

if __name__ == '__main__':
    import nose
    nose.run(argv=['', __file__])





# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
3D plotting of neuroimaging volumes.
"""
from .viz_tools.maps_3d import affine_img_src, plot_map_3d, plot_anat_3d,\
    demo_plot_map_3d




# -*- Mode: Python -*-  Not really, but the syntax is close enough


"""
Incremental (Kalman-like) filters for linear regression. 

Author: Alexis Roche, 2008.
"""

__version__ = '0.1'

# Includes
from fff cimport *

# Exports from fff_glm_kalman.h
cdef extern from "fff_glm_kalman.h":

    ctypedef struct fff_glm_KF:
        size_t t
        size_t dim
        fff_vector* b
        fff_matrix* Vb
        double ssd
        double s2
        double dof
        double s2_corr

    ctypedef struct fff_glm_RKF:
        size_t t
        size_t dim
        fff_glm_KF* Kfilt
        fff_vector* db
        fff_matrix* Hssd
        double spp
        fff_vector* Gspp
        fff_matrix* Hspp
        fff_vector* b
        fff_matrix* Vb
        double s2
        double a
        double dof
        double s2_cor
        fff_vector* vaux
        fff_matrix* Maux

    fff_glm_KF* fff_glm_KF_new(size_t dim)
    void fff_glm_KF_delete(fff_glm_KF* thisone)
    void fff_glm_KF_reset(fff_glm_KF* thisone)
    void fff_glm_KF_iterate(fff_glm_KF* thisone, double y, fff_vector* x)
    fff_glm_RKF* fff_glm_RKF_new(size_t dim)
    void fff_glm_RKF_delete(fff_glm_RKF* thisone)
    void fff_glm_RKF_reset(fff_glm_RKF* thisone)
    void fff_glm_RKF_iterate(fff_glm_RKF* thisone, unsigned int nloop, 
                             double y, fff_vector* x, 
                             double yy, fff_vector* xx)
    void fff_glm_KF_fit(fff_glm_KF* thisone, fff_vector* y, fff_matrix* X)
    void fff_glm_RKF_fit(fff_glm_RKF* thisone, unsigned int nloop, 
                         fff_vector* y, fff_matrix* X)


# Initialize numpy
fffpy_import_array()
import_array()
import numpy as np

# Standard Kalman filter

def ols(ndarray Y, ndarray X, int axis=0): 
    """
    (beta, norm_var_beta, s2, dof) = ols(Y, X, axis=0).

    Ordinary least-square multiple regression using the Kalman filter.
    Fit the N-dimensional array Y along the given axis in terms of the
    regressors in matrix X. The regressors must be stored columnwise.

    OUTPUT: a four-element tuple
    beta -- array of parameter estimates
    norm_var_beta -- normalized variance matrix of the parameter
    estimates (data independent)
    s2 -- array of squared scale
    parameters to multiply norm_var_beta for the variance matrix of
    beta.
    dof -- scalar degrees of freedom.

    REFERENCE:  Roche et al, ISBI 2004.
    """
    cdef fff_vector *y, *b, *s2
    cdef fff_matrix *x
    cdef fff_glm_KF *kfilt
    cdef size_t p
    cdef fffpy_multi_iterator* multi
    cdef double dof

    # View on design matrix
    x = fff_matrix_fromPyArray(X)

    # Number of regressors 
    p = x.size2
    
    # Allocate output arrays B and S2
    #
    # Using Cython cimport of numpy, Y.shape is a C array of npy_intp
    # type; see:
    # http://codespeak.net/pipermail/cython-dev/2009-April/005229.html
    dims = [Y.shape[i] for i in range(Y.ndim)]
    dims[axis] = p
    B = np.zeros(dims, dtype=np.double)
    dims[axis] = 1
    S2 = np.zeros(dims, dtype=np.double)

    # Allocate local structure
    kfilt = fff_glm_KF_new(p)

    # Create a new array iterator 
    multi = fffpy_multi_iterator_new(3, axis, <void*>Y, <void*>B, <void*>S2)

    # Create views
    y = multi.vector[0]
    b = multi.vector[1]
    s2 = multi.vector[2]

    # Loop 
    while(multi.index < multi.size):
        fff_glm_KF_fit(kfilt, y, x)
        fff_vector_memcpy(b, kfilt.b)
        s2.data[0] = kfilt.s2
        fffpy_multi_iterator_update(multi)

    # Normalized variance (computed from the last item)
    VB = fff_matrix_const_toPyArray(kfilt.Vb);    
    dof = kfilt.dof
    
    # Free memory
    fff_matrix_delete(x)
    fff_glm_KF_delete(kfilt)
    fffpy_multi_iterator_delete(multi)

    # Return
    return B, VB, S2, dof
    

def ar1(ndarray Y, ndarray X, int niter=2, int axis=0):
    """
    (beta, norm_var_beta, s2, dof, a) = ar1(Y, X, niter=2, axis=0)

    Refined Kalman filter -- enhanced Kalman filter to account for
    noise autocorrelation using an AR(1) model. Pseudo-likelihood
    multiple regression using the refined Kalman filter, a Kalman
    variant based on a AR(1) error model.  Fit the N-dimensional array
    Y along the given axis in terms of the regressors in matrix X. The
    regressors must be stored columnwise.

    OUTPUT: a five-element tuple
    beta -- array of parameter estimates
    norm_var_beta -- array of normalized variance matrices (which are data dependent
    unlike in standard OLS regression)
    s2 -- array of squared scale parameters to multiply norm_var_beta for the variance matrix of beta.
    dof -- scalar degrees of freedom
    a -- array of error autocorrelation estimates

    REFERENCE:
    Roche et al, MICCAI 2004.
    """
    cdef fff_vector *y, *b, *vb, *s2, *a
    cdef fff_vector Vb_flat
    cdef fff_matrix *x
    cdef fff_glm_RKF *rkfilt
    cdef size_t p, p2
    cdef fffpy_multi_iterator* multi
    cdef double dof

    # View on design matrix
    x = fff_matrix_fromPyArray(X)

    # Number of regressors 
    p = x.size2
    p2 = p*p
    
    # Allocate output arrays B and S2.
    #
    # Using Cython cimport of numpy, Y.shape is a C array of npy_intp
    # type; see:
    # http://codespeak.net/pipermail/cython-dev/2009-April/005229.html
    dims = [Y.shape[i] for i in range(Y.ndim)]
    dims[axis] = p
    B = np.zeros(dims, dtype=np.double)
    dims[axis] = p2
    VB = np.zeros(dims, dtype=np.double)
    dims[axis] = 1
    S2 = np.zeros(dims, dtype=np.double)
    A = np.zeros(dims, dtype=np.double)
 
    # Allocate local structure
    rkfilt = fff_glm_RKF_new(p)

    # Create a new array iterator 
    multi = fffpy_multi_iterator_new(5, axis, <void*>Y, <void*>B, <void*>VB, <void*>S2, <void*>A)

    # Create views
    y = multi.vector[0]
    b = multi.vector[1]
    vb = multi.vector[2]
    s2 = multi.vector[3]
    a = multi.vector[4]

    # Loop 
    while(multi.index < multi.size):
        fff_glm_RKF_fit(rkfilt, niter, y, x)
        fff_vector_memcpy(b, rkfilt.b)
        Vb_flat = fff_vector_view(rkfilt.Vb.data, p2, 1) # rkfilt.Vb contiguous by construction
        fff_vector_memcpy(vb, &Vb_flat)
        s2.data[0] = rkfilt.s2
        a.data[0] = rkfilt.a
        fffpy_multi_iterator_update(multi)

    # Dof 
    dof = rkfilt.dof
    
    # Free memory
    fff_matrix_delete(x)
    fff_glm_RKF_delete(rkfilt) 
    fffpy_multi_iterator_delete(multi)
    
    # Reshape variance array
    dims[axis] = p
    dims.insert(axis+1, p)
    VB = VB.reshape(dims)

    # Return
    return B, VB, S2, dof, A
    






#!/usr/bin/env python
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
from __future__ import print_function # Python 2/3 compatibility
__doc__ = """
Example of a script that perfoms histogram analysis of an activation
image, to estimate activation Z-score with various heuristics:

   * Gamma-Gaussian model
   * Gaussian mixture model
   * Empirical normal null

This example is based on a (simplistic) simulated image.

Needs matplotlib
"""
# Author : Bertrand Thirion, Gael Varoquaux 2008-2009
print(__doc__)

import numpy as np

try:
    import matplotlib.pyplot as plt
except ImportError:
    raise RuntimeError("This script needs the matplotlib library")

import nipy.labs.utils.simul_multisubject_fmri_dataset as simul
import nipy.algorithms.statistics.empirical_pvalue as en

###############################################################################
# simulate the data
shape = (60, 60)
pos = 2 * np.array([[6, 7], [10, 10], [15, 10]])
ampli = np.array([3, 4, 4])

dataset = simul.surrogate_2d_dataset(n_subj=1, shape=shape, pos=pos,
                                     ampli=ampli, width=10.0).squeeze()

fig = plt.figure(figsize=(12, 10))
plt.subplot(3, 3, 1)
plt.imshow(dataset, cmap=plt.cm.hot)
plt.colorbar()
plt.title('Raw data')

Beta = dataset.ravel().squeeze()

###############################################################################
# fit Beta's histogram with a Gamma-Gaussian mixture
gam_gaus_pp = en.gamma_gaussian_fit(Beta, Beta)
gam_gaus_pp = np.reshape(gam_gaus_pp, (shape[0], shape[1], 3))

plt.figure(fig.number)
plt.subplot(3, 3, 4)
plt.imshow(gam_gaus_pp[..., 0], cmap=plt.cm.hot)
plt.title('Gamma-Gaussian mixture,\n first component posterior proba.')
plt.colorbar()
plt.subplot(3, 3, 5)
plt.imshow(gam_gaus_pp[..., 1], cmap=plt.cm.hot)
plt.title('Gamma-Gaussian mixture,\n second component posterior proba.')
plt.colorbar()
plt.subplot(3, 3, 6)
plt.imshow(gam_gaus_pp[..., 2], cmap=plt.cm.hot)
plt.title('Gamma-Gaussian mixture,\n third component posterior proba.')
plt.colorbar()

###############################################################################
# fit Beta's histogram with a mixture of Gaussians
alpha = 0.01
gaus_mix_pp = en.three_classes_GMM_fit(Beta, None, alpha, prior_strength=100)
gaus_mix_pp = np.reshape(gaus_mix_pp, (shape[0], shape[1], 3))


plt.figure(fig.number)
plt.subplot(3, 3, 7)
plt.imshow(gaus_mix_pp[..., 0], cmap=plt.cm.hot)
plt.title('Gaussian mixture,\n first component posterior proba.')
plt.colorbar()
plt.subplot(3, 3, 8)
plt.imshow(gaus_mix_pp[..., 1], cmap=plt.cm.hot)
plt.title('Gaussian mixture,\n second component posterior proba.')
plt.colorbar()
plt.subplot(3, 3, 9)
plt.imshow(gaus_mix_pp[..., 2], cmap=plt.cm.hot)
plt.title('Gamma-Gaussian mixture,\n third component posterior proba.')
plt.colorbar()

###############################################################################
# Fit the null mode of Beta with an empirical normal null

efdr = en.NormalEmpiricalNull(Beta)
emp_null_fdr = efdr.fdr(Beta)
emp_null_fdr = emp_null_fdr.reshape(shape)

plt.subplot(3, 3, 3)
plt.imshow(1 - emp_null_fdr, cmap=plt.cm.hot)
plt.colorbar()
plt.title('Empirical FDR\n ')
plt.show()



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Test the discrete_domain utilities.

Caveat assumes that the MNI template image is available at
in ~/.nipy/tests/data

In those tests, we often access some ROI directly by a fixed index
instead of using the utility functions such as get_id() or select_id().

"""

import numpy as np
from numpy.testing import assert_equal

from ..hroi import HROI_as_discrete_domain_blobs, make_hroi_from_subdomain
from ..mroi import subdomain_from_array
from ..discrete_domain import domain_from_binary_array

shape = (5, 6, 7)


def make_domain():
    """Create a multiple ROI instance
    """
    labels = np.ones(shape)
    dom = domain_from_binary_array(labels, affine=None)
    return dom


#######################################################################
# Test on hierarchical ROI
#######################################################################

def make_hroi(empty=False):
    """Create a multiple ROI instance
    """
    labels = np.zeros(shape)
    if not empty:
        labels[4:, 5:, 6:] = 1
        labels[:2, 0:2, 0:2] = 2
        labels[:2, 5:, 6:] = 3
        labels[:2, 0:2, 6:] = 4
        labels[4:, 0:2, 6:] = 5
        labels[4:, 0:2, 0:2] = 6
        labels[4:, 5:, 0:2] = 7
        labels[:2, 5:, 0:2] = 8
        parents = np.zeros(9)
    else:
        labels = -np.ones(shape)
        parents = np.array([])

    sd = subdomain_from_array(labels, affine=None, nn=0)
    hroi = make_hroi_from_subdomain(sd, parents)
    return hroi


def test_hroi():
    """
    """
    hroi = make_hroi()
    assert_equal(hroi.k, 9)


def test_hroi_isleaf():
    """ Test basic construction of a tree of isolated leaves
    """
    hroi = make_hroi()
    hroi.select_roi([0] + range(2, 9))
    assert_equal(hroi.k, 8)


def test_hroi_isleaf_2():
    """Test tree pruning, with parent remapping
    """
    hroi = make_hroi()
    #import pdb; pdb.set_trace()
    hroi.select_roi(range(1, 9))
    assert_equal(hroi.parents, np.arange(8).astype(np.int))


def test_asc_merge():
    """ Test ascending merge
    """
    hroi = make_hroi()
    s1 = hroi.get_size(0) + hroi.get_size(1)
    total_size = np.sum([hroi.get_size(id) for id in hroi.get_id()])
    assert_equal(hroi.get_size(0, ignore_children=False), total_size)
    hroi.merge_ascending([1])
    assert_equal(hroi.get_size(0), s1)


def test_asc_merge_2():
    """ Test ascending merge

    Test that ROI being their own parent are inchanged.
    """
    hroi = make_hroi()
    s1 = hroi.get_size(0)
    hroi.merge_ascending([0])
    assert_equal(hroi.k, 9)
    assert_equal(hroi.get_size(0), s1)


def test_asc_merge_3():
    """Test ascending merge
    """
    hroi = make_hroi()
    hroi.set_roi_feature('labels', np.arange(9))
    hroi.set_roi_feature('labels2', np.arange(9))
    hroi.merge_ascending([1], pull_features=['labels2'])
    assert_equal(hroi.get_roi_feature('labels', 0), 0)
    assert_equal(hroi.get_roi_feature('labels2', 0), 1)


def test_asc_merge_4():
    """Test ascending merge

    """
    hroi = make_hroi()
    hroi.set_roi_feature('labels', range(9))
    hroi.set_roi_feature('labels2', range(9))
    parents = np.arange(9) - 1
    parents[0] = 0
    hroi.parents = parents
    labels3 = [hroi.label[hroi.label == k] for k in range(hroi.k)]
    hroi.set_feature('labels3', labels3)
    hroi.merge_ascending([1], pull_features=['labels2'])
    assert_equal(hroi.k, 8)
    assert_equal(hroi.get_roi_feature('labels', 0), 0)
    assert_equal(hroi.get_roi_feature('labels2', 0), 1)
    assert_equal(len(hroi.get_feature('labels3')), hroi.k)
    assert_equal(hroi.get_roi_feature('labels2').size, hroi.k)


def test_desc_merge():
    """ Test descending merge
    """
    hroi = make_hroi()
    parents = np.arange(hroi.k)
    parents[1] = 0
    hroi.parents = parents
    s1 = hroi.get_size(0) + hroi.get_size(1)
    hroi.merge_descending()
    assert_equal(hroi.get_size()[0], s1)


def test_desc_merge_2():
    """ Test descending merge
    """
    hroi = make_hroi()
    parents = np.arange(-1, hroi.k - 1)
    parents[0] = 0
    hroi.parents = parents
    hroi.set_roi_feature('labels', np.arange(hroi.k))
    labels2 = [hroi.label[hroi.label == k] for k in range(hroi.k)]
    hroi.set_feature('labels2', labels2)
    hroi.merge_descending()
    assert_equal(hroi.k, 1)
    assert_equal(len(hroi.get_feature('labels2')), hroi.k)
    assert_equal(hroi.get_roi_feature('labels').size, hroi.k)


def test_desc_merge_3():
    """ Test descending merge
    """
    hroi = make_hroi()
    parents = np.minimum(np.arange(1, hroi.k + 1), hroi.k - 1)
    hroi.parents = parents
    hroi.merge_descending()
    assert_equal(hroi.k, 1)


def test_leaves():
    """ Test leaves
    """
    hroi = make_hroi()
    size = hroi.get_size()[1:].copy()
    lroi = hroi.copy()
    lroi.reduce_to_leaves()
    assert_equal(lroi.k, 8)
    assert_equal(lroi.get_size(), size)
    assert_equal(lroi.get_leaves_id(), np.arange(1, 9))


def test_leaves_empty():
    """Test the reduce_to_leaves method on an HROI containing no node

    """
    hroi = make_hroi(empty=True)
    lroi = hroi.reduce_to_leaves()
    assert_equal(lroi.k, 0)


def test_hroi_from_domain():
    dom = make_domain()
    data = np.random.rand(*shape)
    data[:2, 0:2, 0:2] = 2
    rdata = np.reshape(data, (data.size, 1))
    hroi = HROI_as_discrete_domain_blobs(dom, rdata, threshold=1., smin=0)
    assert_equal(hroi.k, 1)


def test_sd_representative():
    """Test the computation of representative features
    """
    hroi = make_hroi()
    hroi.parents = np.arange(9)
    hroi.parents[2] = 1
    data = [[k] * hroi.get_size(k) for k in hroi.get_id()]
    hroi.set_feature('data', data)
    sums = hroi.representative_feature('data')
    for k in hroi.get_id():
        assert_equal(sums[hroi.select_id(k)], k)
    sums2 = hroi.representative_feature('data', ignore_children=False)
    for k in hroi.get_id():
        if k != 1:
            assert_equal(sums2[hroi.select_id(k)], k)
        else:
            assert_equal(sums2[1], 17. / 9)


if __name__ == "__main__":
    import nose
    nose.run(argv=['', __file__])



# -*- Mode: Python -*-  Not really, but the syntax is close enough


"""
Routines for massively univariate random-effect and mixed-effect analysis.

Author: Alexis Roche, 2008.
"""

__version__ = '0.1'


# Includes
from fff cimport *

# Exports from fff_onesample_stat.h
cdef extern from "fff_onesample_stat.h":

  ctypedef enum fff_onesample_stat_flag:
    FFF_ONESAMPLE_EMPIRICAL_MEAN = 0
    FFF_ONESAMPLE_EMPIRICAL_MEDIAN = 1
    FFF_ONESAMPLE_STUDENT = 2
    FFF_ONESAMPLE_LAPLACE = 3
    FFF_ONESAMPLE_TUKEY = 4
    FFF_ONESAMPLE_SIGN_STAT = 5
    FFF_ONESAMPLE_WILCOXON = 6
    FFF_ONESAMPLE_ELR = 7 
    FFF_ONESAMPLE_GRUBB = 8
    FFF_ONESAMPLE_EMPIRICAL_MEAN_MFX = 10,
    FFF_ONESAMPLE_EMPIRICAL_MEDIAN_MFX = 11,
    FFF_ONESAMPLE_STUDENT_MFX = 12,
    FFF_ONESAMPLE_SIGN_STAT_MFX = 15,
    FFF_ONESAMPLE_WILCOXON_MFX = 16, 
    FFF_ONESAMPLE_ELR_MFX = 17, 
    FFF_ONESAMPLE_GAUSSIAN_MEAN_MFX = 19

  ctypedef struct fff_onesample_stat:
    pass

  ctypedef struct fff_onesample_stat_mfx:
    unsigned int niter
    unsigned int constraint
    

  fff_onesample_stat* fff_onesample_stat_new(size_t n, fff_onesample_stat_flag flag, double base)
  void fff_onesample_stat_delete(fff_onesample_stat* thisone)
  double fff_onesample_stat_eval(fff_onesample_stat* thisone, fff_vector* x)

  fff_onesample_stat_mfx* fff_onesample_stat_mfx_new(size_t n, fff_onesample_stat_flag flag, double base) 
  void fff_onesample_stat_mfx_delete(fff_onesample_stat_mfx* thisone) 
  double fff_onesample_stat_mfx_eval(fff_onesample_stat_mfx* thisone, fff_vector* x, fff_vector* vx)

  void fff_onesample_stat_mfx_pdf_fit(fff_vector* w, fff_vector* z,
                                      fff_onesample_stat_mfx* thisone, 
                                      fff_vector* x, fff_vector* vx)
  void fff_onesample_stat_gmfx_pdf_fit(double* mu, double* v, 
                                       fff_onesample_stat_mfx* thisone, 
                                       fff_vector* x, fff_vector* vx)

  void fff_onesample_permute_signs(fff_vector* xx, fff_vector* x, double magic)

# Initialize numpy
fffpy_import_array()
import_array()
import numpy as np


# Stat dictionary
stats = {'mean': FFF_ONESAMPLE_EMPIRICAL_MEAN,
         'median': FFF_ONESAMPLE_EMPIRICAL_MEDIAN,
         'student': FFF_ONESAMPLE_STUDENT,
         'laplace': FFF_ONESAMPLE_LAPLACE,
         'tukey': FFF_ONESAMPLE_TUKEY,
         'sign': FFF_ONESAMPLE_SIGN_STAT,
         'wilcoxon': FFF_ONESAMPLE_WILCOXON,
         'elr': FFF_ONESAMPLE_ELR,
         'grubb': FFF_ONESAMPLE_GRUBB,
         'mean_mfx': FFF_ONESAMPLE_EMPIRICAL_MEAN_MFX,
         'median_mfx': FFF_ONESAMPLE_EMPIRICAL_MEDIAN_MFX,
         'mean_gauss_mfx': FFF_ONESAMPLE_GAUSSIAN_MEAN_MFX,
         'student_mfx': FFF_ONESAMPLE_STUDENT_MFX, 
         'sign_mfx': FFF_ONESAMPLE_SIGN_STAT_MFX,
         'wilcoxon_mfx': FFF_ONESAMPLE_WILCOXON_MFX,
         'elr_mfx': FFF_ONESAMPLE_ELR_MFX}


# Test stat without mixed-effect correction
def stat(ndarray Y, id='student', double base=0.0,
         int axis=0, ndarray Magics=None):
  """
  T = stat(Y, id='student', base=0.0, axis=0, magics=None).
  
  Compute a one-sample test statistic over a number of deterministic
  or random permutations. 
  """
  cdef fff_vector *y, *t, *magics, *yp
  cdef fff_onesample_stat* stat
  cdef fff_onesample_stat_flag flag_stat = stats[id]
  cdef unsigned int n
  cdef unsigned long int simu, nsimu, idx
  cdef double magic
  cdef fffpy_multi_iterator* multi

  # Get number of observations
  n = <unsigned int>Y.shape[axis]

  # Read out magic numbers
  if Magics == None:
    magics = fff_vector_new(1)
    magics.data[0] = 0 ## Just to make sure
  else:
    magics = fff_vector_fromPyArray(Magics)
    
  # Create output array
  nsimu = magics.size
  dims = [Y.shape[i] for i in range(Y.ndim)]
  dims[axis] = nsimu 
  T = np.zeros(dims)

  # Create local structure
  stat = fff_onesample_stat_new(n, flag_stat, base)
  yp = fff_vector_new(n)

  # Multi-iterator 
  multi = fffpy_multi_iterator_new(2, axis, <void*>Y, <void*>T)

  # Vector views
  y = multi.vector[0]
  t = multi.vector[1]
  
  # Loop
  for simu from 0 <= simu < nsimu:
    
    # Set the magic number
    magic = magics.data[simu*magics.stride]
      
    # Reset the multi-iterator
    fffpy_multi_iterator_reset(multi); 
    
    # Perform the loop 
    idx = simu*t.stride
    while(multi.index < multi.size):
      fff_onesample_permute_signs(yp, y, magic)
      t.data[idx] = fff_onesample_stat_eval(stat, yp)
      fffpy_multi_iterator_update(multi)

  # Free memory 
  fffpy_multi_iterator_delete(multi)
  fff_vector_delete(yp)
  fff_vector_delete(magics)
  fff_onesample_stat_delete(stat)

  # Return
  return T


def stat_mfx(ndarray Y, ndarray V, id='student_mfx', double base=0.0,
             int axis=0, ndarray Magics=None, unsigned int niter=5):
  """
  T = stat_mfx(Y, V, id='student_mfx', base=0.0, axis=0, magics=None, niter=5).
  
  Compute a one-sample test statistic, with mixed-effect correction,
  over a number of deterministic or random permutations.
  """
  cdef fff_vector *y, *v, *t, *magics, *yp
  cdef fff_onesample_stat_mfx* stat
  cdef fff_onesample_stat_flag flag_stat = stats[id]
  cdef int n
  cdef unsigned long int nsimu_max, simu, idx
  cdef double magic
  cdef fffpy_multi_iterator* multi

  # Get number of observations
  n = <int>Y.shape[axis]

  # Read out magic numbers
  if Magics == None:
    magics = fff_vector_new(1)
    magics.data[0] = 0 ## Just to make sure
  else:
    magics = fff_vector_fromPyArray(Magics)
    
  # Create output array
  nsimu = magics.size
  dims = [Y.shape[i] for i in range(Y.ndim)]
  dims[axis] = nsimu 
  T = np.zeros(dims)

  # Create local structure
  stat = fff_onesample_stat_mfx_new(n, flag_stat, base)
  stat.niter = niter
  yp = fff_vector_new(n)

  # Multi-iterator 
  multi = fffpy_multi_iterator_new(3, axis, <void*>Y, <void*>V, <void*>T)

  # Vector views
  y = multi.vector[0]
  v = multi.vector[1]
  t = multi.vector[2]
  
  # Loop  
  for simu from 0 <= simu < nsimu:

    # Set the magic number
    magic = magics.data[simu*magics.stride]
    
    # Reset the multi-iterator      
    fffpy_multi_iterator_reset(multi)
    
    # Perform the loop 
    idx = simu*t.stride
    while(multi.index < multi.size):
      fff_onesample_permute_signs(yp, y, magic)
      t.data[idx] = fff_onesample_stat_mfx_eval(stat, yp, v)
      fffpy_multi_iterator_update(multi)
      

  # Free memory
  fffpy_multi_iterator_delete(multi)
  fff_vector_delete(yp)
  fff_vector_delete(magics)
  fff_onesample_stat_mfx_delete(stat)
  
  # Return
  return T



def pdf_fit_mfx(ndarray Y, ndarray V, int axis=0, int niter=5, int constraint=0, double base=0.0):
  """
  (W, Z) = pdf_fit_mfx(data=Y, vardata=V, axis=0, niter=5, constraint=False, base=0.0).
  
  Comments to follow.
  """
  cdef fff_vector *y, *v, *w, *z
  cdef fff_onesample_stat_mfx* stat
  cdef fffpy_multi_iterator* multi
  cdef int n = Y.shape[axis]

  # Create output array
  dims = [Y.shape[i] for i in range(Y.ndim)]
  W = np.zeros(dims)
  Z = np.zeros(dims)

  # Create local structure
  stat = fff_onesample_stat_mfx_new(n, FFF_ONESAMPLE_EMPIRICAL_MEAN_MFX, base)
  stat.niter = niter
  stat.constraint = constraint

  # Multi-iterator 
  multi = fffpy_multi_iterator_new(4, axis, <void*>Y, <void*>V, <void*>W, <void*>Z)

  # Create views on nd-arrays
  y = multi.vector[0]
  v = multi.vector[1]
  w = multi.vector[2]
  z = multi.vector[3]

  # Loop
  while(multi.index < multi.size):
    fff_onesample_stat_mfx_pdf_fit(w, z, stat, y, v)
    fffpy_multi_iterator_update(multi)
  

  # Delete local structures
  fffpy_multi_iterator_delete(multi)
  fff_onesample_stat_mfx_delete(stat)

  # Return
  return W, Z


def pdf_fit_gmfx(ndarray Y, ndarray V, int axis=0, int niter=5, int constraint=0, double base=0.0):
  """
  (MU, S2) = pdf_fit_gmfx(data=Y, vardata=V, axis=0, niter=5, constraint=False, base=0.0).
  
  Comments to follow.
  """
  cdef fff_vector *y, *v, *mu, *s2
  cdef fff_onesample_stat_mfx* stat
  cdef fffpy_multi_iterator* multi
  cdef int n = Y.shape[axis]
  
  # Create output array
  dims = [Y.shape[i] for i in range(Y.ndim)]
  dims[axis] = 1
  MU = np.zeros(dims)
  S2 = np.zeros(dims)

  # Create local structure
  stat = fff_onesample_stat_mfx_new(n, FFF_ONESAMPLE_STUDENT_MFX, base)
  stat.niter = niter
  stat.constraint = constraint

  # Multi-iterator 
  multi = fffpy_multi_iterator_new(4, axis, <void*>Y, <void*>V, <void*>MU, <void*>S2)

  # Create views on nd-arrays
  y = multi.vector[0]
  v = multi.vector[1]
  mu = multi.vector[2]
  s2 = multi.vector[3]

  # Loop
  while(multi.index < multi.size):
    fff_onesample_stat_gmfx_pdf_fit(mu.data, s2.data, stat, y, v)
    fffpy_multi_iterator_update(multi)
  

  # Delete local structures
  fffpy_multi_iterator_delete(multi)
  fff_onesample_stat_mfx_delete(stat)

  # Return
  return MU, S2

  



#!/usr/bin/env python

import numpy as np
import numpy.random as nr
from unittest import TestCase

from ..bipartite_graph import (BipartiteGraph, cross_knn, cross_eps, 
                               check_feature_matrices)


def basicdata():
    x = np.array( [[-1.998,-2.024], [-0.117,-1.010], [1.099,-0.057],
                   [ 1.729,-0.252], [1.003,-0.021], [1.703,-0.739],
                   [-0.557,1.382],[-1.200,-0.446],[-0.331,-0.256],
                   [-0.800,-1.584]])
    return x

def test_feature_matrices():
    """ test that feature matrices are correctly checked
    """
    x, y = nr.rand(10, 1), nr.rand(12)
    check_feature_matrices(x, y)
    check_feature_matrices(y, x)
    check_feature_matrices(x, x)
    check_feature_matrices(y, y)

def test_cross_knn_1():
    """ test the construction of k-nn bipartite graph
    """
    x = basicdata()
    G = cross_knn(x, x, 2)
    assert (G.E == 20)
        
def test_cross_knn_2():
    """ test the construction of k-nn bipartite graph
    """
    x = basicdata()
    G = cross_knn(x, x, 1)
    assert (G.E == 10)
    
def test_cross_eps_1():
    """ test the construction of eps-nn bipartite graph
    """
    x = basicdata()
    y = x + 0.1 * nr.randn(x.shape[0], x.shape[1])
    G = cross_eps(x, y, 1.)
    D = G.weights
    assert((D < 1).all())

def test_copy():
    """ test that the weighted graph copy is OK
    """
    x = basicdata()
    G = cross_knn(x, x, 2)
    K = G.copy()
    assert K.edges.shape == (20, 2)
    
def test_subraph_left():
    """ Extraction of the 'left subgraph'
    """
    x = basicdata()
    g = cross_knn(x, x, 2)
    valid = np.arange(10) < 7
    sl = g.subgraph_left(valid)
    assert sl.V == 7
    assert sl.W == 10
    assert sl.edges[:, 0].max() == 6
    
def test_subraph_left2():
    """ Extraction of the 'left subgraph', without renumb=False
    """
    x = basicdata()
    g = cross_knn(x, x, 2)
    valid = np.arange(10) < 7
    sl = g.subgraph_left(valid, renumb=False)
    assert sl.V == 10
    assert sl.W == 10
    assert sl.edges[:, 0].max() == 6

def test_subraph_right():
    """ Extraction of the 'right subgraph'
    """
    x = basicdata()
    g = cross_knn(x, x, 2)
    valid = np.arange(10) < 7
    sr = g.subgraph_right(valid)
    assert sr.W == 7
    assert sr.V == 10
    assert sr.edges[:, 1].max() == 6

def test_subraph_right2():
    """ Extraction of the 'right subgraph', with renumb = False
    """
    x = basicdata()
    g = cross_knn(x, x, 2)
    valid = np.arange(10) < 7
    sr = g.subgraph_right(valid, renumb = False)
    assert sr.W == 10
    assert sr.V == 10
    assert sr.edges[:, 1].max() == 6
    


if __name__ == '__main__':
    import nose
    nose.run(argv=['', __file__])





# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
CoordinateSystems are used to represent the space in which the image resides.

A CoordinateSystem contains named coordinates, one for each dimension
and a coordinate dtype.  The purpose of the CoordinateSystem is to
specify the name and order of the coordinate axes for a particular
space.  This allows one to compare two CoordinateSystems to determine
if they are equal.

"""
__docformat__ = 'restructuredtext'

import numpy as np

class CoordinateSystemError(Exception):
    pass


class CoordinateSystem(object):
    """An ordered sequence of named coordinates of a specified dtype.

    A coordinate system is defined by the names of the coordinates,
    (attribute ``coord_names``) and the numpy dtype of each coordinate
    value (attribute ``coord_dtype``).  The coordinate system can also
    have a name.

    >>> names = ['first', 'second', 'third']
    >>> cs = CoordinateSystem(names, 'a coordinate system', np.float)
    >>> cs.coord_names
    ('first', 'second', 'third')
    >>> cs.name
    'a coordinate system'
    >>> cs.coord_dtype
    dtype('float64')

    The coordinate system also has a ``dtype`` which is the composite
    numpy dtype, made from the (``names``, ``coord_dtype``).

    >>> dtype_template = [(name, np.float) for name in cs.coord_names]
    >>> dtype_should_be = np.dtype(dtype_template)
    >>> cs.dtype == dtype_should_be
    True

    Two CoordinateSystems are equal if they have the same dtype
    and the same names and the same name.

    >>> another_cs = CoordinateSystem(names, 'not irrelevant', np.float)
    >>> cs == another_cs
    False
    >>> cs.dtype == another_cs.dtype
    True
    >>> cs.name == another_cs.name
    False
    """

    _doc = {}

    name = 'world-LPI'
    _doc['name'] = 'Name describing the CoordinateSystem'

    coord_names = ('x', 'y', 'z')
    _doc['coord_names'] = 'Tuple of names describing each coordinate.'

    coord_dtype = np.float64
    _doc['coord_dtype'] = 'The builtin, scalar,  dtype of each coordinate.'

    ndim = 3
    _doc['ndim'] = 'The number of dimensions'

    dtype = np.dtype([('x', np.float),
                      ('y', np.float),
                      ('z', np.float)])
    _doc['dtype'] = 'The composite dtype of the CoordinateSystem, ' + \
                    'expresses the fact that there are three numbers, the' + \
                    'first one corresponds to "x" and the second to "y".'

    def __init__(self, coord_names, name='', coord_dtype=np.float):
        """Create a coordinate system with a given name and coordinate names.

        The CoordinateSystem has two dtype attributes:

        #. self.coord_dtype is the dtype of the individual coordinate values
        #. self.dtype is the recarray dtype for the CoordinateSystem
           which combines the coord_names and the coord_dtype.  This
           functions as the description of the CoordinateSystem.

        Parameters
        ----------
        coord_names : iterable
           A sequence of coordinate names.
        name : string, optional
           The name of the coordinate system
        coord_dtype : np.dtype, optional
           The dtype of the coord_names.  This should be a built-in
           numpy scalar dtype. (default is np.float).  The value can
           by anything that can be passed to the np.dtype constructor.
           For example ``np.float``, ``np.dtype(np.float)`` or ``f8``
           all result in the same ``coord_dtype``.

        Examples
        --------
        >>> c = CoordinateSystem('ij', name='input')
        >>> print c
        CoordinateSystem(coord_names=('i', 'j'), name='input', coord_dtype=float64)
        >>> c.coord_dtype
        dtype('float64')
        """
        # this allows coord_names to be an iterator and have a length
        coord_names = tuple(coord_names)
        # Make sure each coordinate is unique
        if len(set(coord_names)) != len(coord_names):
            raise ValueError('coord_names must have distinct names')
        # verify that the dtype is coord_dtype for sanity
        sctypes = (np.sctypes['int'] + np.sctypes['float'] +
                   np.sctypes['complex'] + np.sctypes['uint'] + [np.object])
        coord_dtype = np.dtype(coord_dtype)
        if coord_dtype not in sctypes:
            raise ValueError('Coordinate dtype should be one of %s' % sctypes)
        # Set all the attributes
        self.name = name
        self.coord_names = coord_names
        self.coord_dtype = coord_dtype
        self.ndim = len(coord_names)
        self.dtype = np.dtype([(name, self.coord_dtype)
                               for name in self.coord_names])

    # All attributes are read only

    def __setattr__(self, key, value):
        if key in self.__dict__:
            raise AttributeError('the value of %s has already been set and all attributes are read-only' % key)
        object.__setattr__(self, key, value)

    def index(self, coord_name):
        """Return the index of a given named coordinate.

        >>> c = CoordinateSystem('ij', name='input')
        >>> c.index('i')
        0
        >>> c.index('j')
        1
        """
        return list(self.coord_names).index(coord_name)

    def __ne__(self, other):
        return not self.__eq__(other)

    def __eq__(self, other):
        """Equality is defined by self.dtype and self.name

        Parameters
        ----------
        other : :class:`CoordinateSystem`
           The object to be compared with

        Returns
        -------
        tf: bool
        """
        return (self.dtype == other.dtype) and (self.name == other.name)

    def similar_to(self, other):
        """Similarity is defined by self.dtype, ignoring name

        Parameters
        ----------
        other : :class:`CoordinateSystem`
           The object to be compared with

        Returns
        -------
        tf: bool
        """
        return (self.dtype == other.dtype)

    def __repr__(self):
        """Create a string representation of the coordinate system

        Returns
        -------
        s : string
        """
        return ("CoordinateSystem(coord_names=%s, name='%s', coord_dtype=%s)" %
                (self.coord_names, self.name, self.coord_dtype))


    def _checked_values(self, arr):
        ''' Check ``arr`` for valid dtype and shape as coordinate values.

        Raise Errors for failed checks.

        The dtype of ``arr`` has to be castable (without loss of precision) to
        ``self.coord_dtype``.  We use numpy ``can_cast`` for this check.

        The last (or only) axis of ``arr`` should be of length ``self.ndim``.

        Parameters
        ----------
        arr : array-like
            array to check

        Returns
        -------
        checked_arr : array
            Possibly reshaped array

        Examples
        --------
        >>> cs = CoordinateSystem('ijk', coord_dtype=np.float32)
        >>> arr = np.array([1, 2, 3], dtype=np.int16)
        >>> cs._checked_values(arr) # 1D is OK with matching dimensions 
        array([[1, 2, 3]], dtype=int16)
        >>> cs._checked_values(arr.reshape(1,3)) # as is 1 by N
        array([[1, 2, 3]], dtype=int16)

        This next is the wrong shape:

        >>> cs._checked_values(arr.reshape(3,1)) #doctest: +IGNORE_EXCEPTION_DETAIL
        Traceback (most recent call last):
           ...
        CoordinateSystemError: Array shape[-1] (1) must match CoordinateSystem ndim (3).
          CoordinateSystem(coord_names=('i', 'j', 'k'), name='', coord_dtype=float32)

        Wrong length:

        >>> cs._checked_values(arr[0:2]) #doctest: +IGNORE_EXCEPTION_DETAIL
        Traceback (most recent call last):
           ...
        CoordinateSystemError: Array shape[-1] (2) must match CoordinateSystem ndim (3).
          CoordinateSystem(coord_names=('i', 'j', 'k'), name='', coord_dtype=float32)

        The dtype has to be castable:

        >>> cs._checked_values(np.array([1, 2, 3], dtype=np.float64)) #doctest: +IGNORE_EXCEPTION_DETAIL
        Traceback (most recent call last):
           ...
        CoordinateSystemError: Cannot cast array dtype float64 to CoordinateSystem coord_dtype float32.
          CoordinateSystem(coord_names=('i', 'j', 'k'), name='', coord_dtype=float32)

        The input array is unchanged, even if a reshape has
        occurred. The returned array points to the same data.

        >>> checked = cs._checked_values(arr)
        >>> checked.shape == arr.shape
        False
        >>> checked is arr
        False
        >>> arr[0]
        1
        >>> checked[0,0] = 10
        >>> arr[0]
        10

        For a 1D CoordinateSystem, passing a 1D vector length N could be a
        mistake (you were expecting an N-dimensional coordinate system), or it
        could be N points in 1D.  Because it is ambiguous, this is an error.

        >>> cs = CoordinateSystem('x')
        >>> cs._checked_values(1)
        array([[1]])
        >>> cs._checked_values([1, 2]) #doctest: +IGNORE_EXCEPTION_DETAIL
        Traceback (most recent call last):
           ...
        CoordinateSystemError: Array shape[-1] (2) must match CoordinateSystem ndim (1).
          CoordinateSystem(coord_names=('x',), name='', coord_dtype=float64)

        But of course 2D, N by 1 is OK

        >>> cs._checked_values(np.array([1,2,3]).reshape(3, 1))
        array([[1],
               [2],
               [3]])
        '''
        arr = np.atleast_2d(arr)
        if arr.shape[-1] != self.ndim:
            raise CoordinateSystemError('Array shape[-1] (%s) must match '
                                        'CoordinateSystem ndim (%d).\n  %s'
                                        % (arr.shape[-1], self.ndim, str(self)))
        if not np.can_cast(arr.dtype, self.coord_dtype):
            raise CoordinateSystemError('Cannot cast array dtype %s to '
                                        'CoordinateSystem coord_dtype %s.\n  %s' %
                                        (arr.dtype, self.coord_dtype, str(self)))
        return arr.reshape((-1, self.ndim))


def is_coordsys(obj):
    """ Test if `obj` has the CoordinateSystem API

    Parameters
    ----------
    obj : object
        Object to test

    Returns
    -------
    tf : bool
        True if `obj` has the coordinate system API

    Examples
    --------
    >>> is_coordsys(CoordinateSystem('xyz'))
    True
    >>> is_coordsys(CoordSysMaker('ikj'))
    False
    """
    if not hasattr(obj, 'coord_names'):
        return False
    if not hasattr(obj, 'name'):
        return False
    if not hasattr(obj, 'coord_dtype'):
        return False
    # Distinguish from CoordSysMaker
    return not callable(obj)


def safe_dtype(*dtypes):
    """Determine a dtype to safely cast all of the given dtypes to.

    Safe dtypes are valid numpy dtypes or python types which can be
    cast to numpy dtypes.  See numpy.sctypes for a list of valid
    dtypes.  Composite dtypes and string dtypes are not safe dtypes.

    Parameters
    ----------
    dtypes : sequence of ``np.dtype``

    Returns
    -------
    dtype : np.dtype

    Examples
    --------
    >>> c1 = CoordinateSystem('ij', 'input', coord_dtype=np.float32)
    >>> c2 = CoordinateSystem('kl', 'input', coord_dtype=np.complex)
    >>> safe_dtype(c1.coord_dtype, c2.coord_dtype)
    dtype('complex128')

    >>> # Strings are invalid dtypes
    >>> safe_dtype(type('foo'))
    Traceback (most recent call last):
    ...
    TypeError: dtype must be valid numpy dtype int, uint, float, complex or object

    >>> # Check for a valid dtype
    >>> myarr = np.zeros(2, np.float32)
    >>> myarr.dtype.isbuiltin
    1

    >>> # Composite dtypes are invalid
    >>> mydtype = np.dtype([('name', 'S32'), ('age', 'i4')])
    >>> myarr = np.zeros(2, mydtype)
    >>> myarr.dtype.isbuiltin
    0
    >>> safe_dtype(mydtype)
    Traceback (most recent call last):
    ...
    TypeError: dtype must be valid numpy dtype int, uint, float, complex or object
    """
    arrays = [np.zeros(2, dtype) for dtype in dtypes]
    kinds = [a.dtype.kind for a in arrays]
    if not set(kinds).issubset('iubfcO'):
        raise TypeError('dtype must be valid numpy dtype '
                        'int, uint, float, complex or object')
    return np.array(arrays).dtype


def product(*coord_systems, **kwargs):
    """Create the product of a sequence of CoordinateSystems.

    The coord_dtype of the result will be determined by ``safe_dtype``.

    Parameters
    ----------
    \*coord_systems : sequence of :class:`CoordinateSystem`
    name : str
        Name of ouptut coordinate system

    Returns
    -------
    product_coord_system : :class:`CoordinateSystem`

    Examples
    --------
    >>> c1 = CoordinateSystem('ij', 'input', coord_dtype=np.float32)
    >>> c2 = CoordinateSystem('kl', 'input', coord_dtype=np.complex)
    >>> c3 = CoordinateSystem('ik', 'in3')

    >>> print product(c1, c2)
    CoordinateSystem(coord_names=('i', 'j', 'k', 'l'), name='product', coord_dtype=complex128)

    >>> print product(c1, c2, name='another name')
    CoordinateSystem(coord_names=('i', 'j', 'k', 'l'), name='another name', coord_dtype=complex128)

    >>> product(c2, c3)
    Traceback (most recent call last):
       ...
    ValueError: coord_names must have distinct names
    """
    name = kwargs.pop('name', 'product')
    if kwargs:
        raise TypeError('Unexpected kwargs %s' % kwargs)
    coords = []
    for c in coord_systems:
        coords += c.coord_names
    dtype = safe_dtype(*[c.coord_dtype for c in coord_systems])
    return CoordinateSystem(coords, name, coord_dtype=dtype)


class CoordSysMakerError(Exception):
    pass


class CoordSysMaker(object):
    """ Class to create similar coordinate maps of different dimensions
    """
    coord_sys_klass = CoordinateSystem

    def __init__(self, coord_names, name='', coord_dtype=np.float):
        """Create a coordsys maker with given axis `coord_names`

        Parameters
        ----------
        coord_names : iterable
           A sequence of coordinate names.
        name : string, optional
           The name of the coordinate system
        coord_dtype : np.dtype, optional
           The dtype of the coord_names.  This should be a built-in
           numpy scalar dtype. (default is np.float).  The value can
           by anything that can be passed to the np.dtype constructor.
           For example ``np.float``, ``np.dtype(np.float)`` or ``f8``
           all result in the same ``coord_dtype``.

        Examples
        --------
        >>> cmkr = CoordSysMaker('ijk', 'a name')
        >>> print cmkr(2)
        CoordinateSystem(coord_names=('i', 'j'), name='a name', coord_dtype=float64)
        >>> print cmkr(3)
        CoordinateSystem(coord_names=('i', 'j', 'k'), name='a name', coord_dtype=float64)
        """
        self.coord_names = tuple(coord_names)
        self.name = name
        self.coord_dtype = coord_dtype

    def __call__(self, N, name=None, coord_dtype=None):
        """ Create coordinate system of length `N`

        Parameters
        ----------
        N : int
            length of coordinate map
        name : None or str, optional
            Name of coordinate map.  Default is ``self.name``
        coord_dtype : None or dtype
            ``coord_dtype`` of returned coordinate system.  Default is
            ``self.coord_dtype``

        Returns
        -------
        csys : coordinate system
        """
        if name is None:
            name = self.name
        if coord_dtype is None:
            coord_dtype = self.coord_dtype
        if N > len(self.coord_names):
            raise CoordSysMakerError('Not enough axis names (have %d, '
                                     'you asked for %d)' %
                                     (len(self.coord_names), N))
        return self.coord_sys_klass(self.coord_names[:N], name, coord_dtype)


def is_coordsys_maker(obj):
    """ Test if `obj` has the CoordSysMaker API

    Parameters
    ----------
    obj : object
        Object to test

    Returns
    -------
    tf : bool
        True if `obj` has the coordinate system API

    Examples
    --------
    >>> is_coordsys_maker(CoordSysMaker('ikj'))
    True
    >>> is_coordsys_maker(CoordinateSystem('xyz'))
    False
    """
    if not hasattr(obj, 'coord_names'):
        return False
    if not hasattr(obj, 'name'):
        return False
    if not hasattr(obj, 'coord_dtype'):
        return False
    # Distinguish from CoordinateSystem
    return callable(obj)



import numpy as np
from numpy.testing import assert_almost_equal, assert_equal, TestCase

from ..hemodynamic_models import (
    spm_hrf, spm_time_derivative, spm_dispersion_derivative,
    resample_regressor, _orthogonalize, sample_condition,
    _regressor_names, _hrf_kernel, glover_hrf, 
    glover_time_derivative, compute_regressor)


def test_spm_hrf():
    """ test that the spm_hrf is correctly normalized and has correct length
    """
    h = spm_hrf(2.0)
    assert_almost_equal(h.sum(), 1)
    assert len(h) == 256

def test_spm_hrf_derivative():
    """ test that the spm_hrf is correctly normalized and has correct length
    """
    h = spm_time_derivative(2.0)
    assert_almost_equal(h.sum(), 0)
    assert len(h) == 256
    h = spm_dispersion_derivative(2.0)
    assert_almost_equal(h.sum(), 0)
    assert len(h) == 256

def test_glover_hrf():
    """ test that the spm_hrf is correctly normalized and has correct length
    """
    h = glover_hrf(2.0)
    assert_almost_equal(h.sum(), 1)
    assert len(h) == 256

def test_glover_time_derivative():
    """ test that the spm_hrf is correctly normalized and has correct length
    """
    h = glover_time_derivative(2.0)
    assert_almost_equal(h.sum(), 0)
    assert len(h) == 256
    
def test_resample_regressor():
    """ test regressor resampling on a linear function
    """
    x = np.linspace(0, 1, 200)
    y = np.linspace(0, 1, 30)
    z = resample_regressor(x, x, y)
    assert_almost_equal(z, y)

def test_resample_regressor_nl():
    """ test regressor resampling on a sine function
    """
    x = np.linspace(0, 10, 1000)
    y = np.linspace(0, 10, 30)
    z = resample_regressor(np.cos(x), x, y)
    assert_almost_equal(z, np.cos(y), decimal=2)

def test_orthogonalize():
    """ test that the orthogonalization is OK 
    """
    X = np.random.randn(100, 5)
    X = _orthogonalize(X)
    K = np.dot(X.T, X)
    K -= np.diag(np.diag(K))
    assert (K ** 2).sum() < 1.e-16

def test_orthogonalize_trivial():
    """ test that the orthogonalization is OK 
    """
    X = np.random.randn(100)
    Y = X.copy()
    X = _orthogonalize(X)
    assert (Y == X).all()

def test_sample_condition_1():
    """ Test that the experimental condition is correctly sampled
    """
    condition = ([1, 20, 36.5], [0, 0, 0], [1, 1, 1])
    frametimes = np.linspace(0, 49, 50)
    reg, rf = sample_condition(condition, frametimes, oversampling=1)
    assert reg.sum() == 3
    assert reg[1] == 1
    assert reg[37] == 1
    assert reg[20] ==1

def test_sample_condition_2():
    """ Test that the experimental condition is correctly sampled
    """
    condition = ([1, 20, 36.5], [2, 2, 2], [1, 1, 1])
    frametimes = np.linspace(0, 49, 50)
    reg, rf = sample_condition(condition, frametimes, oversampling=1)
    assert reg.sum() == 6
    assert reg[1] == 1
    assert reg[38] == 1
    assert reg[21] ==1

def test_sample_condition_3():
    """ Test that the experimental condition is correctly sampled
    """
    condition = ([1, 20, 36.5], [2, 2, 2], [1, 1, 1])
    frametimes = np.linspace(0, 49, 50)
    reg, rf = sample_condition(condition, frametimes, oversampling=10)
    assert_almost_equal(reg.sum(), 60.)
    assert reg[10] == 1
    assert reg[380] == 1
    assert reg[210] == 1
    assert np.sum(reg > 0) == 60
    
def test_sample_condition_4():
    """ Test that the experimental condition is correctly sampled
    with wrongly placed trials
    """
    condition = ([-3, 1, 20, 36.5, 51], [0, 0, 0, 0, 0], [1, 1, 1, 1, 1])
    frametimes = np.linspace(0, 49, 50)
    reg, rf = sample_condition(condition, frametimes, oversampling=1)
    assert reg.sum() == 3
    assert reg[1] == 1
    assert reg[37] == 1
    assert reg[20] ==1

def test_sample_condition_5():
    """ Test that the experimental condition is correctly sampled
    """
    condition = ([1, 20, 36.5], [2, 2, 2], [1., -1., 5.])
    frametimes = np.linspace(0, 49, 50)
    reg, rf = sample_condition(condition, frametimes, oversampling=1)
    assert reg.sum() == 10
    assert reg[1] == 1.
    assert reg[20] == -1.
    assert reg[37] == 5.

def test_names():
    """ Test the regressor naming function
    """
    name = 'con'
    assert _regressor_names(name, 'spm') == ['con']
    assert _regressor_names(name, 'spm_time') == ['con', 'con_derivative']
    assert _regressor_names(name, 'spm_time_dispersion') == \
        ['con', 'con_derivative', 'con_dispersion']
    assert _regressor_names(name, 'canonical') == ['con']
    assert _regressor_names(name, 'canonical with derivative') == \
        ['con', 'con_derivative']

def test_hkernel():
    """ test the hrf computation
    """
    tr = 2.0
    h = _hrf_kernel('spm', tr)
    assert_almost_equal(h[0], spm_hrf(tr))
    assert len(h) == 1
    h = _hrf_kernel('spm_time', tr)
    assert_almost_equal(h[1], spm_time_derivative(tr))
    assert len(h) == 2
    h = _hrf_kernel('spm_time_dispersion', tr)
    assert_almost_equal(h[2], spm_dispersion_derivative(tr))
    assert len(h) == 3
    h = _hrf_kernel('canonical', tr)
    assert_almost_equal(h[0], glover_hrf(tr))
    assert len(h) == 1
    h = _hrf_kernel('canonical with derivative', tr)
    assert_almost_equal(h[1], glover_time_derivative(tr))
    assert_almost_equal(h[0], glover_hrf(tr))
    assert len(h) == 2
    h = _hrf_kernel('fir', tr, fir_delays = np.arange(4))
    assert len(h) == 4
    for dh in h:
        assert dh.sum() == 16.
    
def test_make_regressor_1():
    """ test the generated regressor
    """
    condition = ([1, 20, 36.5], [2, 2, 2], [1, 1, 1])
    frametimes = np.linspace(0, 69, 70)
    hrf_model = 'spm'
    reg, reg_names = compute_regressor(condition, hrf_model, frametimes)
    assert_almost_equal(reg.sum(), 6, 1)
    assert reg_names[0] == 'cond'

def test_make_regressor_2():
    """ test the generated regressor
    """
    condition = ([1, 20, 36.5], [0, 0, 0], [1, 1, 1])
    frametimes = np.linspace(0, 69, 70)
    hrf_model = 'spm'
    reg, reg_names = compute_regressor(condition, hrf_model, frametimes)
    assert_almost_equal(reg.sum() * 16, 3, 1)
    assert reg_names[0] == 'cond'


def test_make_regressor_3():
    """ test the generated regressor
    """
    condition = ([1, 20, 36.5], [0, 0, 0], [1, 1, 1])
    frametimes = np.linspace(0, 138, 70)
    hrf_model = 'fir'
    reg, reg_names = compute_regressor(condition, hrf_model, frametimes, 
                                       fir_delays=np.arange(4))
    assert (np.unique(reg) == np.array([0, 1])).all()
    assert (np.sum(reg, 0) == np.array([3, 3, 3, 3])).all()
    assert len(reg_names) == 4

if __name__ == "__main__":
    import nose
    nose.run(argv=['', __file__])



#!/usr/bin/env python

import numpy as np
import numpy.random as nr
from ..ggmixture import GGGM, GGM, Gamma
import scipy.stats as st

def test_GGM1(verbose=0):
    shape = 1 
    scale = 1
    mean = 0
    var = 1
    G = GGM(shape,scale,mean,var)
    sx = 1000
    x = -2.5 + nr.randn(sx)
    G.estimate(x)
    b = np.absolute(G.mean+2.5)<0.5
    if verbose:
        #G.parameters()
        print x.max()
    assert(b)

def test_GGM2(verbose=0):
    shape = 1 
    scale = 1
    mean = 0
    var = 1
    G = GGM(shape,scale,mean,var)
    sx = 1000
    x = -2.5 + nr.randn(sx)
    G.estimate(x)
    if verbose:
        G.parameters()
    b = np.absolute(G.mixt)<0.1
    assert(b)

def test_GGGM0(verbose=0, seed=1):
    G = GGGM()
    sx = 1000
    #x = np.array([float(st.t.rvs(dof)) for i in range(sx)])
    if seed:
        nr = np.random.RandomState([seed])
    else:
        import numpy.random as nr
    x = nr.randn(sx)
    G.init(x)
    G.estimate(x)
    if verbose:
        G.parameters()
    assert(np.absolute(G.mean)<0.3)

def test_GGGM1(verbose=0):
    G = GGGM()
    sx = 10000
    x = np.array([float(st.t.rvs(5)) for i in range(sx)])
    G.init_fdr(x)
    G.estimate(x)
    if verbose:
        G.parameters()
    assert(np.absolute(G.mean)<0.1)
    
def test_GGGM2(verbose=0):
    G = GGGM()
    sx = 10000
    x = nr.randn(sx)
    G.init_fdr(x)
    G.estimate(x)
    assert(G.mixt[1]>0.9)

def test_GGGM3(verbose=0):
    G = GGGM()
    sx = 1000
    x = 100 + np.array([float(st.t.rvs(5)) for i in range(sx)])
    G.init(x)
    G.estimate(x)
    if verbose:
        G.parameters()
    assert(np.absolute(G.mixt[0])<1.e-15)

def test_gamma_parameters1(verbose=0):
    import numpy.random as nr
    n = 1000
    X = nr.gamma(11., 3., n)
    G = Gamma()
    G.estimate(X)
    if verbose:
        G.parameters()
    assert(np.absolute(G.shape-11)<2.)

def test_gamma_parameters2(verbose=0):
    import numpy.random as nr
    n = 1000
    X = nr.gamma(11., 3., n)
    G = Gamma()
    G.estimate(X)
    if verbose:
        G.parameters()
    assert(np.absolute(G.scale-3)<0.5)


if __name__ == '__main__':
    import nose
    nose.run(argv=['', __file__])





""" Generic transform class

This implementation specifies an API.  We've done our best to avoid checking
instances, so any class implementing this API should be valid in the places
(like registration routines) that use transforms.  If that isn't true, it's a
bug.
"""

class Transform(object):
    """ A default transformation class

    This class specifies the tiny API.  That is, the class should implement:

    * obj.param - the transformation exposed as a set of parameters. Changing
      param should change the transformation
    * obj.apply(pts) - accepts (N,3) array-like of points in 3 dimensions,
      returns an (N, 3) array of transformed points
    * obj.compose(xform) - accepts another object implementing ``apply``, and
      returns a new transformation object, where the resulting transformation is
      the composition of the ``obj`` transform onto the ``xform`` transform.
    """
    def __init__(self, func):
        self.func = func

    def apply(self, pts):
        return self.func(pts)

    def compose(self, other):
        return self.__class__(
            lambda pts : self.func(other.apply(pts)))

    @property
    def param(self):
        raise AttributeError('No param for generic transform')



# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
""" Testing diagnostic screen
"""

import numpy as np

import nipy as ni
from ..screens import screen
from ..timediff import time_slice_diffs
from ...utils.pca import pca
from ...utils.tests.test_pca import res2pos1

from nose.tools import (assert_true, assert_false, assert_equal, assert_raises)

from numpy.testing import (assert_array_equal, assert_array_almost_equal,
                           assert_almost_equal)

from nipy.testing import funcfile, anatfile


def test_screen():
    img = ni.load_image(funcfile)
    res = screen(img)
    assert_equal(res['mean'].ndim, 3)
    assert_equal(res['pca'].ndim, 4)
    assert_equal(sorted(res.keys()),
                 ['max', 'mean', 'min',
                  'pca', 'pca_res',
                  'std', 'ts_res'])
    data = img.get_data()

    assert_array_equal(np.max(data, axis=-1), res['max'].get_data())
    assert_array_equal(np.mean(data, axis=-1), res['mean'].get_data())
    assert_array_equal(np.min(data, axis=-1), res['min'].get_data())
    assert_array_equal(np.std(data, axis=-1), res['std'].get_data())
    pca_res = pca(data, axis=-1, standardize=False, ncomp=10)
    # On windows, there seems to be some randomness in the PCA output vector
    # signs; this routine sets the basis vectors to have first value positive,
    # and therefore standardized the signs
    pca_res = res2pos1(pca_res)
    screen_pca_res = res2pos1(res['pca_res'])
    for key in pca_res:
        assert_almost_equal(pca_res[key], screen_pca_res[key])
    ts_res = time_slice_diffs(data)
    for key in ts_res:
        assert_array_equal(ts_res[key], res['ts_res'][key])



""" Testing resample function
"""

import numpy as np

from ....core.image.image_spaces import (as_xyz_image,
                                         xyz_affine)
from ....core.api import Image, vox2mni
from ..resample import resample
from ..affine import Affine

from numpy.testing import assert_array_almost_equal


def _test_resample(arr, interp_orders):
    # Check basic cases of resampling
    img = Image(arr, vox2mni(np.eye(4)))
    T = Affine()
    for i in interp_orders:
        img2 = resample(img, T, interp_order=i)
        assert_array_almost_equal(img2.get_data(), img.get_data())
        img_aff = as_xyz_image(img)
        img2 = resample(img, T, reference=(img_aff.shape, xyz_affine(img_aff)),
                        interp_order=i)
        assert_array_almost_equal(img2.get_data(), img.get_data())


def test_resample_float_data():
    arr = np.random.rand(10, 11, 12)
    _test_resample(arr, (0, 1, 3, 5))

def test_resample_int_data():
    arr = np.random.randint(100, size=(10, 11, 12)) - 50
    _test_resample(arr, (3,))

def test_resample_uint_data():
    arr = np.random.randint(100, size=(10, 11, 12)).astype('uint8')
    img = Image(arr, vox2mni(np.eye(4)))
    T = Affine((.5, .5, .5, .1, .1, .1, 0, 0, 0, 0, 0, 0))
    img2 = resample(img, T)
    assert(np.min(img2.get_data()) >= 0)
    assert(np.max(img2.get_data()) < 255)

